#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <signal.h>
#include <getopt.h>
#include <errno.h>

#include <glib.h>
#include <gtk/gtk.h>

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#if HAVE_MYSQL
#include <mysql.h>
#endif

#include "../libsqirc/sqirc.h"
#include "md5.h"

#undef VERSION
#define VERSION "sqbot - Squaroid Hosting Bot 0.0.1"

#define DEFAULT_DB_USER       "root"
#define DEFAULT_DB_NAME       "squaroid"
#define DEFAULT_DB_PASSWD     "azzit"
#define DEFAULT_DB_HOST       "localhost"

#define SQ_INITIAL_SCORE 100

#define USER_ISOP       1<<0
#define USER_MAYTALK    1<<1
#define USER_RESTORED   1<<2

typedef struct gamedone
{
	char nick[10];
	char opponent[10];
	ulong p1_score;
	ulong p2_score;
	ulong who_am_i;
} Gamedone;

typedef struct user
{
	char  nick[64];
	char  user[64];
	char  host[128];
	char  realname[200];
	char  playerid[33];
	ulong score;
	ulong flags;
	GSList *gamesdone;
} User;

typedef struct channel
{
	char name[200];
	GSList *users;
} Channel;

typedef struct connectiondata
{
	GSList *channels;                    /* channels joined */
	GSList *users;                       /* users known */
} ConnectionData;


static SQIRC_FUNC_CMDCLIENT(cmdclient_join);
static SQIRC_FUNC_CMDCLIENT(cmdclient_part);
static struct client_command client_commands[] =
{
	{ "JOIN",       cmdclient_join },
	{ "PART",       cmdclient_part },
	{ "QUIT",       cmdclient_part },
	LAST_COMMAND
};

static SQIRC_FUNC_CMDCLIENT(cmdctcp_version);
static SQIRC_FUNC_CMDCLIENT(cmdctcp_gamedone);
static SQIRC_FUNC_CMDCLIENT(cmdctcp_restore);
static SQIRC_FUNC_CMDCLIENT(cmdctcp_save);
static struct client_command ctcp_commands[] =
{
	{ "VERSION",    cmdctcp_version },
	{ "GAMEDONE",   cmdctcp_gamedone },
	{ "RESTORE",    cmdctcp_restore },
	{ "SAVE",       cmdctcp_save },
	LAST_COMMAND
};

static SQIRC_FUNC_CMDSERVER(cmdserver_whois);
static SQIRC_FUNC_CMDSERVER(cmdserver_namreply);
static struct server_command server_commands[] =
{
	CMDIGNORE(1),
	CMDIGNORE(2),
	CMDIGNORE(3),
	CMDIGNORE(4),
	CMDIGNORE(437),
	CMDIGNORE(ERR_CHANNELISFULL),
	CMDIGNORE(ERR_INVITEONLYCHAN),
	CMDIGNORE(ERR_BANNEDFROMCHAN),
	CMDIGNORE(ERR_BADCHANNELKEY),
	CMDIGNORE(RPL_LUSERCLIENT),
	CMDIGNORE(RPL_LUSERME),
	CMDIGNORE(RPL_LUSERCHANNELS),
	CMDIGNORE(RPL_LUSERUNKNOWN),
	CMDIGNORE(RPL_UNKNOWN_265),
	CMDIGNORE(RPL_UNKNOWN_266),
	CMDIGNORE(333),
	CMDIGNORE(RPL_TOPIC),
	CMDIGNORE(329),
	CMDIGNORE(RPL_CHANNELMODEIS),
	CMDIGNORE(RPL_LISTEND),
	CMDIGNORE(RPL_AWAY),
	CMDIGNORE(RPL_ISON),
	CMDIGNORE(RPL_WHOISSERVER),
	CMDIGNORE(RPL_WHOWASUSER),
	CMDIGNORE(RPL_WHOISIDLE),
	CMDIGNORE(RPL_ENDOFWHOIS),
	CMDIGNORE(RPL_WHOISOPERATOR),
	CMDIGNORE(RPL_WHOISCHANNELS),
	CMDIGNORE(RPL_LISTSTART),
	CMDIGNORE(RPL_LIST),
	
	CMDIGNORE(RPL_ENDOFWHO),
	{ RPL_WHOISUSER, cmdserver_whois },
	{ RPL_WHOREPLY, cmdserver_whois },
	
	CMDIGNORE(RPL_ENDOFNAMES),
	{ RPL_NAMREPLY, cmdserver_namreply },
	{ 0, NULL }
};

struct preferences
{
	char nick[64];
	char realname[200];
	char host[128];
	char channel[200];
	char db_host[128];
	char db_user[128];
	char db_passwd[128];
	char db_name[128];
} preferences;

static struct connection *connection[5];
static gboolean running = TRUE;

#if HAVE_MYSQL
static MYSQL mysql;
static void sqbot_mysql_store(User *);        /* store score in DB */
static void sqbot_mysql_retrieve(User **);    /* receive userdata from db */
#endif
int sqbot_mysql_query(const char *);

static User *find_user_by_nick(ConnectionData *, const char *);
static Channel *find_channel_by_name(ConnectionData *, const char *);
static Gamedone *find_gamedone_by_nicknames(ConnectionData *, const char *, const char *);

static void sqbot_assign_new_score(User *, User *);
void sqbot_save_user(struct connection *, User *);


/* ************************************************************* */
/*                     client commands                           */
/* ************************************************************* */
SQIRC_FUNC_CMDCLIENT(cmdclient_join)
{
	ConnectionData *cdata;
	User *user = 0;
	Channel *channel = 0;
	GSList *list = 0;

	if (!strcasecmp(data->from.nick, preferences.nick))
		{
			/* we're a bot... no score */
			sqirc_send(connection, "NOTICE %s :UPDATEPLAYER %s 0\r\n",
								 preferences.channel, preferences.nick);
			/* get data about people in this room */
			sqirc_send(connection, "MODE %s\r\n", data->message+1);
			sqirc_send(connection, "WHO %s\r\n", data->message+1);
		}
	else
		sqirc_send(connection, "WHOIS %s\r\n", data->from.nick);
	
	cdata = (ConnectionData *)connection->data;
	printf("*** %s@%s has joined %s\n",
				 data->from.nick, data->from.host, data->message+1);
	
	if ( (channel = find_channel_by_name(cdata, data->message+1)) == 0 )
		{
			channel = malloc(sizeof(Channel));
			bzero(channel, sizeof(Channel));
			snprintf(channel->name, 200, "%s", data->message+1);
			
			cdata->channels = g_slist_append(cdata->channels, channel);
			printf("+++ channel %s created in memory\n", channel->name);
		}
	
	if ( (find_user_by_nick(cdata, data->from.nick)) == 0)
		{
			user = malloc(sizeof(User));
			bzero(user, sizeof(User));
			strncpy(user->nick, data->from.nick, 64);
			strncpy(user->user, data->from.user, 64);
			strncpy(user->host, data->from.host, 128);
			user->score = SQ_INITIAL_SCORE;

			/* we're a bot... no score */
			if (!strcasecmp(data->from.nick, preferences.nick))
				user->score = 0;
			
			cdata->users = g_slist_append(cdata->users, user);
			channel->users = g_slist_append(channel->users, user);
			printf("+++ user %s created in memory\n", user->nick);
		}

	list = cdata->users;
	while (list)
		{
			User *player = (User *)list->data;
			list = list->next;
			
			if (!strcasecmp(player->nick, user->nick))
				continue;
			/*
				else if (!strcasecmp(player->nick, preferences.nick))
				continue;
			*/
			if (player->score != 100)
				sqirc_send(connection, "NOTICE %s :UPDATEPLAYER %s %d\r\n",
									 user->nick, player->nick, player->score);
		}
	
}

SQIRC_FUNC_CMDCLIENT(cmdclient_part)
{
	ConnectionData *cdata = (ConnectionData *)connection->data;
	User *user = find_user_by_nick(cdata, data->from.nick);
	
	/* if the score has been restored, also save it ! no cheating :) */
	if ( (user->flags & USER_RESTORED) == USER_RESTORED)
		sqbot_save_user(connection, user);
	
	cdata->users = g_slist_remove(cdata->users, user);
	printf("+++ user %s left. memory freed\n", user->nick);
	free(user);
}

/* ************************************************************* */
/*                       ctcp commands                           */
/* ************************************************************* */
SQIRC_FUNC_CMDCLIENT(cmdctcp_version)
{
	sqirc_send(connection, "NOTICE %s :\001VERSION " VERSION "\001\r\n",
						 data->from.nick);
}

SQIRC_FUNC_CMDCLIENT(cmdctcp_gamedone)
{
	ConnectionData *cdata = (ConnectionData *)connection->data;
	Gamedone *game;
	User *user;
	
	if ( (game = find_gamedone_by_nicknames(cdata,
																					data->from.nick,
																					data->list[1])) == 0)
		game = find_gamedone_by_nicknames(cdata,
																			data->list[1],
																			data->from.nick);
	
	if (game == 0)
		{
			game = malloc(sizeof(Gamedone));
			bzero(game, sizeof(Gamedone));
			
			strncpy(game->nick, data->from.nick, 10);
			strncpy(game->opponent, data->list[1], 10);
			
			game->p1_score = atoi(data->list[2]);
			game->p2_score = atoi(data->list[3]);
			game->who_am_i = atoi(data->list[4]);
			
			user = find_user_by_nick(cdata, data->from.nick);
			printf("+++ new gamedata added: %s against %s\n",
						 game->nick, game->opponent);
			user->gamesdone = g_slist_append(user->gamesdone, game);
			return;
		}
	else
		{
			User *opponent = find_user_by_nick(cdata, game->opponent);

			user = find_user_by_nick(cdata, game->nick);
			
			if (user->gamesdone)
				user->gamesdone = g_slist_remove(user->gamesdone, game);
			if (opponent->gamesdone)
				opponent->gamesdone = g_slist_remove(opponent->gamesdone, game);
			
			switch(game->who_am_i)
				{
				case 1:
					if (game->p1_score > game->p2_score)
						sqbot_assign_new_score(user, opponent);
					else
						sqbot_assign_new_score(opponent, user);
					break;
				case 2:
					if (game->p2_score > game->p1_score)
						sqbot_assign_new_score(user, opponent);
					else
						sqbot_assign_new_score(opponent, user);
					break;
				default:
					break;
				}

			sqirc_send(connection, "NOTICE %s :UPDATEPLAYER %s %d\r\n",
								 preferences.channel, user->nick, user->score);
			sqirc_send(connection, "NOTICE %s :UPDATEPLAYER %s %d\r\n",
								 preferences.channel, opponent->nick, opponent->score);
			printf("+++ removing gamedata %s against %s\n",
						 game->nick, game->opponent);
			free(game);
		}
}

SQIRC_FUNC_CMDCLIENT(cmdctcp_restore)
{
#if HAVE_MYSQL
	MYSQL_RES *result;
	MYSQL_ROW row;
	
	User *user = find_user_by_nick((ConnectionData *)connection->data,
																 data->from.nick);
	char buffer[2000];
	
	if (user == 0)
		return;

	if ( (user->flags & USER_RESTORED) == USER_RESTORED )
		{	
			sqirc_send(connection,
								 "PRIVMSG %s :sorry. your scores have already been restored for this session!\r\n",
								 data->from.nick);
			return;
		}
	
	snprintf(user->playerid, 200, "%s", data->list[1]);
	snprintf(buffer, 2000, "select score from scores where pid = '%s'",
					 user->playerid);
	sqbot_mysql_query(buffer);
	result = mysql_store_result(&mysql);
	if ( (row = mysql_fetch_row(result)) )
		{
			user->score = strtol(row[0], NULL, 10);
			sqirc_send(connection,
								 "PRIVMSG %s :found your entry. sending the announcement\r\n",
								 data->from.nick);
			sqirc_send(connection, "NOTICE %s :UPDATEPLAYER %s %d\r\n",
								 preferences.channel, user->nick, user->score);
			user->flags |= USER_RESTORED;
		}
	else
		{
			sqirc_send(connection,
								 "PRIVMSG %s :sorry. could not find your database entry!\r\n",
								 data->from.nick);
		}
	mysql_free_result(result);
#else
	sqirc_send(connection,
						 "PRIVMSG %s :this bot does not have database support\r\n",
						 data->from.nick);
#endif
}

SQIRC_FUNC_CMDCLIENT(cmdctcp_save)
{
#if HAVE_MYSQL
	User *user;
	char buffer[2000];
	
	user = find_user_by_nick((ConnectionData *)connection->data,
													 data->from.nick);
	
	sqbot_save_user(connection, user);
	
#else
	sqirc_send(connection,
						 "PRIVMSG %s :this bot does not have database support\r\n",
						 data->from.nick);
#endif
}

/* ************************************************************* */
/*                     server commands                           */
/* ************************************************************* */
SQIRC_FUNC_CMDSERVER(cmdserver_whois)
{
	ConnectionData *cdata = (ConnectionData *)connection->data;
	User *user;
	char *realname = data->message;
	char *nick;
	
	if (reply == RPL_WHOISUSER)
		nick = data->list[0];
	else
		nick = data->list[4];
	
	while (*realname != ':' && *realname != 0)
		realname++;
	
	if (reply != RPL_WHOISUSER)
		while (*realname != ' ' && *realname != 0)
			realname++;
	
	if ( (user = find_user_by_nick(cdata, nick)) != 0)
		{
			strncpy(user->realname, ++realname, 200);
			if (reply == RPL_WHOISUSER)
				{
					bzero(user->user, 64);
					strncpy(user->user, data->list[1], 64);
					bzero(user->host, 128);
					strncpy(user->host, data->list[2], 128);
				}

			if (strstr(user->realname , "Squaroid Host"))
				{
					sqirc_send(connection, "MODE %s +o %s\r\n",
										 preferences.channel, user->nick);
					printf("==> setting MODE +o for %s\n", user->nick);
				}
			printf("+++ updated %s's realname to \"%s\"\n",
						 nick, user->realname);
		}
	
}

SQIRC_FUNC_CMDSERVER(cmdserver_namreply)
{
	ConnectionData *cdata = (ConnectionData *)connection->data;
	User *user = 0;
	ulong flags = 0;
	int i;

	for (i = 2; data->list[i]; i++)
		{
			char *name = data->list[i];
			
			while (*name == '@' || *name == '+' || *name == ':')
				{
					switch(*name)
						{
						case '@': flags |= USER_ISOP;    break;
						case '+': flags |= USER_MAYTALK; break;
						}
					name++;
				}
			
			if ( (find_user_by_nick(cdata, name)) == 0)
				{
					user = malloc(sizeof(User));
					bzero(user, sizeof(User));
					strncpy(user->nick, name, 64);
					user->score = SQ_INITIAL_SCORE;
					
					cdata->users = g_slist_append(cdata->users, user);
					printf("+++ user %s created in memory\n", user->nick);
				}

		}
}

/* ************************************************************* */
/* ************************************************************* */

void sig_int()
{
	fprintf(stderr, "Caught SIGINT. Terminating!\n");
	running = FALSE;
}

User *find_user_by_nick(ConnectionData *data, const char *nick)
{
	GSList *list = data->users;
	
	while (list)
		{
			User *user = (User *)list->data;
			if (!strcasecmp(user->nick, nick))
				return user;
			list = list->next;
		}
	return 0;
}

Channel *find_channel_by_name(ConnectionData *data, const char *name)
{
	GSList *list = data->channels;

	while (list)
		{
			Channel *channel = (Channel *)list->data;
			if (!strcasecmp(channel->name, name))
				return channel;
			list = list->next;
		}
	return 0;
}

Gamedone *find_gamedone_by_nicknames(ConnectionData *data,
																		 const char *nickname,
																		 const char *opponent)
{
	User *user;
	GSList *list;
	
	user = find_user_by_nick(data, nickname);
	list = user->gamesdone;
	while (list)
		{
			Gamedone *game = (Gamedone *)list->data;
			if (!strcasecmp(game->nick, nickname)
					&& !strcasecmp(game->opponent, opponent))
				return game;
			list = list->next;
		}
	return 0;
}

/* assign new scores to winner and loser after a game */
void sqbot_assign_new_score(User *winner, User *loser)
{
	double diff;
	double quot;
	
	/*
		diff is evaluated by the following equation:
		
		diff = ( Ls / Ws) * 8
		
		Ls : Loser-Score
		Ws : Winner-Score
		if diff is lower than 1 or higher than 16, it's adjusted for 
		these values
	*/
	
	quot = (double)((double)loser->score / (double)winner->score);
	diff = (quot * 8.00);
	
	if (diff < 1)
		diff = 1;
	else if (diff > 16)
		diff = 16;
	winner->score += diff;
	loser->score -= diff;
	
}

void sqbot_save_user(struct connection *connection, User *user)
{
	char buffer[2000];
	
	if (user == 0)
		return;
	
	if (user->playerid[0] == 0)
		{
			md5_state_t state;
			md5_byte_t digest[16];
			char random[32];
			char buf[3];
			int di;
			FILE *id_fd;              /* file descriptor */
			
			id_fd = fopen("/dev/urandom", "r");
			srand(fgetc(id_fd));
			fgets(random, 31, id_fd);
			fclose(id_fd);
			
			md5_init(&state);
			md5_append(&state, (const md5_byte_t *)user->nick, strlen(user->nick));
			md5_append(&state, (const md5_byte_t *)random, 32);
			md5_finish(&state, digest);
			
			bzero(user->playerid, 33);
			for (di = 0; di < 16; ++di)
				{
					snprintf(buf, 3, "%02x", digest[di]);
					strcat(user->playerid, buf);
				}
			printf("+++ created id '%s' for user %s\n", user->playerid, user->nick);
			sqirc_send(connection, "NOTICE %s :PLAYERID %s\r\n",
								 user->nick, user->playerid);
			snprintf(buffer, 2000,
							 "insert into scores (pid, score) values ('%s',%ld)",
							 user->playerid, user->score);
		}
	else
		{
			snprintf(buffer, 2000, "update scores set score = %ld where pid = '%s'",
							 user->score, user->playerid);
		}

	sqbot_mysql_query(buffer);
}

int sqbot_mysql_query(const char *query)
{
#if HAVE_MYSQL
	int res = mysql_query(&mysql, query);
	if (res != 0 && mysql_error(&mysql)[0] != 0)
		fprintf(stderr, "Mysql error: '%s'\nThe query was -%s-",
						mysql_error(&mysql), query);
	
	return res;
#else
	return 0;
#endif
}

void sqbot_cleanup_simple_glist(GSList *list)
{
	while (list)
		{
			void *tmp = list->data;
			list = g_slist_remove(list, tmp);
			free(tmp);
		}
}

void sqbot_connection_cleanup(struct connection *connection)
{
	ConnectionData *data = (ConnectionData *)connection->data;

	if (data != 0)
		{
			while (data->users)
				{
					User *user = (User *)data->users->data;
					sqbot_cleanup_simple_glist(user->gamesdone);
					data->users = g_slist_remove(data->users, user);
					free(user);
				}

			while (data->channels)
				{
					Channel *channel = (Channel *)data->channels->data;
					g_slist_free(channel->users);
					data->channels = g_slist_remove(data->channels, channel);
					free(channel);
				}

			
			sqbot_cleanup_simple_glist(data->users);
			free(connection->data);
			connection->data = 0;
		}
	
	if (connection->status == IRC_CONNECTED)
		sqirc_send(connection, "QUIT :squaroid host terminated. cya!\r\n");
}

SQIRC_FUNC_ERRORHANDLER(sqbot_error_handler)
{
	fprintf(stderr, "> ");
	switch(irc_error)
		{
			/* for debug messages like network traffic, no data is present
				 so we'll have to take the command pointer */
		case IRCDEBUG_NET_IN:
			fprintf(stderr, "[IN] %s\n", command);
			break;
		case IRCDEBUG_NET_OUT:
			fprintf(stderr, "[OUT] %s\n", command);
			break;
		case IRCERR_UNHANDLED_SERVER_CMD:
			fprintf(stderr, "Unhandled server reply: '%d'\n", reply);
			fprintf(stderr, "[IN] %s\n", data->buffer);
			break;
		case IRCERR_UNHANDLED_CLIENT_CMD:
			fprintf(stderr, "Unhandled client command: '%s'\n", command);
			fprintf(stderr, "[IN] %s\n", data->buffer);
			break;
		case IRCERR_UNHANDLED_CTCP_CMD:
			fprintf(stderr, "Unhandled CTCP command: '%s'\n", command);
			fprintf(stderr, "[IN] %s\n", data->buffer);
			break;
		default:
			fprintf(stderr, "[libsqirc:ERR] %lx: %s\n", irc_error, command);
			break;
		}
}

extern int optind, opterr, optopt;
extern char *optarg;
int main(int argc, char **argv)
{
	struct irc_userdata user;
	ConnectionData *conndata;
	ulong flags = SQIRC_SKIP_MOTD;
	int c, ret;
	
	printf(VERSION "\n");
	
	bzero(&preferences, sizeof(struct preferences));
	snprintf(preferences.nick, 64,  "Squaroid");
	snprintf(preferences.host, 128, "games.squaroid.org");
	snprintf(preferences.channel, 200, "#squaroid");
	snprintf(preferences.db_host, 200, DEFAULT_DB_HOST);
	snprintf(preferences.db_user, 200, DEFAULT_DB_USER);
	snprintf(preferences.db_name, 200, DEFAULT_DB_NAME);
	snprintf(preferences.db_passwd, 200, DEFAULT_DB_PASSWD);
	
	while ( (c = getopt(argc, argv, "c:DMvh:n:")) != EOF)
		{
			switch(c)
				{
				case 'D':
					flags ^= SQIRC_DEBUG;
					break;
				case 'M':
					flags ^= SQIRC_SKIP_MOTD;
					break;
				case 'c':
					strncpy(preferences.channel, optarg, 200);
					break;
				case 'h':
					strncpy(preferences.host, optarg, 128);
					break;
				case 'v':
					exit(0);
					break;
				case 'n':
					strncpy(preferences.nick, optarg, 64);
					break;
				default: break;
				}
		}
	
	signal(SIGINT, sig_int);

#if HAVE_MYSQL
	mysql_init(&mysql);
	mysql_real_connect(&mysql,
										 preferences.db_host, preferences.db_user,
										 preferences.db_passwd, preferences.db_name,
										 0, NULL, 0);
	if (mysql_error(&mysql)[0] != 0)
		{
			fprintf(stderr, "Mysql error: %s\n", mysql_error(&mysql));
			exit(1);
		}
#endif
	
	sqirc_initialize(server_commands, client_commands, ctcp_commands);
	sqirc_register_callback(IRC_FUNC_CLEANUP, sqbot_connection_cleanup);
	sqirc_register_callback(IRC_FUNC_ERRORHANDLER, sqbot_error_handler);
	
	snprintf(user.username, IRC_BUFUSER, "Squaroid");
	snprintf(user.nickname, IRC_BUFNICK, preferences.nick);
	snprintf(user.desc, IRC_BUFSIZE,  "Squaroid Host IRC-Bot");
	if ( (ret = sqirc_connect(preferences.host,
														IRC_DEFAULT_PORT,
														flags,
														&user,
														&connection[0])) > IRCERR_BASE )
		{
			switch(ret)
				{
				case IRCERR_INVALID_HOSTNAME:
					fprintf(stderr, "*** invalid hostname\n");
					break;
				case IRCERR_LOOKUP_FAIL:
					fprintf(stderr, "*** failed to lookup hostname %s\n",
									preferences.host);
					break;
				case IRCERR_NOSOCKET:
					fprintf(stderr, "*** error creating socket: %s\n", strerror(errno));
					break;
				case IRCERR_NOCONNECTION:
					fprintf(stderr, "*** could not connect to %s\n", preferences.host);
					break;
				default:
					break;
				}
			fprintf(stderr, "+++ an error occured. Exiting.\n\n");
			exit(1);
		}
	conndata = malloc(sizeof(ConnectionData));
	bzero(conndata, sizeof(ConnectionData));
	sqirc_send(connection[0], "JOIN %s\r\n", preferences.channel);
	connection[0]->data = conndata;
	
	while(running)
		gtk_main_iteration();

#if HAVE_MYSQL
	mysql_close(&mysql);
#endif
	
	sqirc_close_all_connections();
	return 0;
}
