#include "squaroid.h"

#define MODE_9ONLY     1              /* only 9 point squares are valid */
#define MODE_16ONLY    2              /* only 16 point squares are valid */
#define MODE_9HIGH     3              /* only 9 or less points squares */
#define MODE_16HIGH    4              /* only 16 or less points squares */

#define XHIGH ((XHighData *)this->data)

typedef struct
{
	int mode;
} XHighData;

/* static Game *current_game = 0; */
static Plugin *this = 0;

ulong xhigh_onload(Plugin *xhigh)
{
	this = xhigh;
	printf("XHigh plugin loaded.\nCopyright (c) 2000 Lukas Schroeder\n");
	return PLUGIN_SUCCESS;
}

ulong xhigh_onunload()
{
	printf("XHigh plugin unloaded\n");
	return PLUGIN_SUCCESS;
}

void xhigh_callback_gamemode(GtkWidget *w, gpointer data)
{
	XHIGH->mode = (int)data;
}

ulong xhigh_oninvite(Game *game, GtkWidget **vbox)
{
	GtkWidget *frame, *frame_box, *button;
	
	if (!XHIGH)
		XHIGH = game_alloc(game, sizeof(XHighData));
	if (!XHIGH)
		fprintf(stderr, "[XHIGH] FATAL ERROR: no plugindata\n");

	*vbox = gtk_vbox_new(FALSE, 5);
	
	frame = gtk_frame_new("Game Variation");
	gtk_box_pack_start(GTK_BOX(*vbox), frame, FALSE, FALSE, 5);
	frame_box = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(frame), frame_box);
	
	button = 0;
	button = gtk_radio_button_new_with_label(RGROUP(button), "9 High");
	gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 xhigh_callback_gamemode, (void*)MODE_9HIGH);
	if (XHIGH && XHIGH->mode == MODE_9HIGH)
		ACTIVATE(button);
	
	button = gtk_radio_button_new_with_label(RGROUP(button), "16 High");
	gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 xhigh_callback_gamemode, (void*)MODE_16HIGH);
	if (XHIGH && XHIGH->mode == MODE_16HIGH)
		ACTIVATE(button);

	button = gtk_radio_button_new_with_label(RGROUP(button), "9 Only");
	gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 xhigh_callback_gamemode, (void*)MODE_9ONLY);
	if (XHIGH && XHIGH->mode == MODE_9ONLY)
		ACTIVATE(button);
	
	button = gtk_radio_button_new_with_label(RGROUP(button), "16 Only");
  gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 xhigh_callback_gamemode, (void*)MODE_16ONLY);
	if (XHIGH && XHIGH->mode == MODE_16ONLY)
		ACTIVATE(button);

	return PLUGIN_SUCCESS;
}

ulong xhigh_oninvitesend(Game *game)
{
	sqirc_send(game->connection,
						 "NOTICE %s :XHIGH %d\r\n",
						 game->opponent,
						 XHIGH->mode);
	
	return PLUGIN_SUCCESS;
}

ulong xhigh_ongamestart(Game *game)
{
	
	if (XHIGH->mode == 0)
		XHIGH->mode = MODE_16HIGH;
	
	game_chatmessage(game, STARTOFF " This is a \0032XHigh\003 Game.\n");
	if (XHIGH->mode == MODE_9ONLY)
		game_chatmessage(game, STARTOFF " Only squares with a value of \0035 9 \003 are counted\n");
	else if (XHIGH->mode == MODE_16ONLY)
		game_chatmessage(game, STARTOFF " Only squares with a value of \0035 16 \003 are counted\n");
	else if (XHIGH->mode == MODE_9HIGH)
		game_chatmessage(game, STARTOFF " Only squares with a value of \0035 9 or less \003 are counted\n");
	else if (XHIGH->mode == MODE_16HIGH)
		game_chatmessage(game, STARTOFF " Only squares with a value of \0035 16 or less \003 are counted\n");
		
	XHIGH = 0;
	return PLUGIN_SUCCESS;
}

ulong xhigh_onsquare(Game *game, int size)
{
  XHighData *data = (XHighData *)game->plugindata;
	
	if (data->mode == MODE_9ONLY && size == 3)
		return 9;                        /* valid square, add 9 to score */
	else if (data->mode == MODE_16ONLY && size == 4)
		return 16;                       /* valid square, add 16 to score */
	else if (data->mode == MODE_9HIGH && size <= 3)
		return (size*size);
	else if (data->mode == MODE_16HIGH && size <= 4)
		return (size*size);
	
	return 0; /* no score */
}

ulong xhigh_ongameover(Game *game)
{
	
	return PLUGIN_FAILURE;        /* we want the original gameover handling */
}

ulong xhigh_oncleanup(Game *game)
{
	return PLUGIN_SUCCESS;
}

SQIRC_FUNC_CMDCLIENT(xhigh_net_xhigh)
{
	Game *game = 0;

	if ( !(game = find_game_by_opponent(connection, data->from.nick)) )
		{
			fprintf(stderr, "[XHigh] Could not find Game by Opponent %s\n",
							data->from.nick);
			return;
		}
	
	XHIGH = game->plugindata;              /* restore the pointer! */
	if (XHIGH == 0)
		XHIGH = game_alloc(game, sizeof(XHighData));
	
	XHIGH->mode = atoi(data->list[1]);
	fprintf(stderr, "[XHIGH] mode set to %d\n", XHIGH->mode);
}

struct client_command xhigh_commands[] =
{
	{ "XHIGH",         xhigh_net_xhigh },
	LAST_COMMAND
};


SquaroidPlugin plugin_xhigh =
{
	"XHigh",                              /* name */
	0,                                    /* flags */
	xhigh_commands,                       /* network command table */
	xhigh_onload,                         /* on load */
	xhigh_onunload,                       /* on unload */
	xhigh_oninvite,                       /* on invite */
	xhigh_oninvitesend,                   /* in invite send */
	xhigh_ongamestart,                    /* on GameStart */
	NULL,                                 /* on move */
	xhigh_onsquare,                       /* on square */
	xhigh_ongameover,                     /* on game over */
	xhigh_oncleanup,                      /* on cleanup */
};

#if COMPILE_DL
SquaroidPlugin *get_module()
{
	return &plugin_xhigh;
}
#endif
