/* Taken from X-Chat and modified to suit Squaroid's needs by Lukas Schrder.
 * Copyright (C) 1998 Peter Zelezny.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "squaroid.h"

char *
cfg_get_str (char *cfg, char *var, char *dest)
{
  *dest = 0;
  while (1)
    {
      if (!strncasecmp (var, cfg, strlen (var)))
	{
	  char *value, t;
	  cfg += strlen (var);
	  while (*cfg == ' ' || *cfg == '=')
	    cfg++;
	  value = cfg;
	  while (*cfg != 0 && *cfg != '\n')
	    cfg++;
	  t = *cfg;
	  *cfg = 0;
	  strcpy (dest, value);
	  *cfg = t;
	  return cfg;
	}
      while (*cfg != 0 && *cfg != '\n')
	cfg++;
      if (*cfg == 0)
	return 0;
      cfg++;
      if (*cfg == 0)
	return 0;
    }
}

static void
cfg_put_str (int fh, char *var, char *value)
{
  char buf[512];

  sprintf (buf, "%s= %s\n", var, value);
  write (fh, buf, strlen (buf));
}

void
cfg_put_int (int fh, unsigned long value, char *var)
{
  char buf[400];

  if (value == -1)
    value = 1;

  sprintf (buf, "%s= %lu\n", var, value);
  write (fh, buf, strlen (buf));
}

unsigned long
cfg_get_int (char *cfg, char *var)
{
  char str[128];

  if (!cfg_get_str (cfg, var, str))
    return 0;

  return atol (str);
}

static char *
default_file (void)
{
  static char file[142];
  sprintf (file, "%s/.squaroid.conf", g_get_home_dir ());
  return (file);
}

void
load_config (int mode)
{
  struct stat st;
  char *cfg, *username;
  int fh;

  username = g_get_user_name ();
  if (mode == 1)
    {
      memset (&preferences, 0, sizeof (struct preferences));

      /* put in default values */
      strncpy (preferences.nickname, username, 63);
      strncpy (preferences.gfxtheme, DEFAULT_GFXTHEME, 64);
      strncpy (preferences.gfxdir, DEFAULT_GFXDIR, 255);
      strncpy (preferences.sfxdir, DEFAULT_SFXDIR, 255);
      strncpy (preferences.hostname, DEFAULT_IRCSERVER, 255);
      preferences.port = 6667;
      preferences.saveonexit = 0;
      preferences.sound = 1;

      if (!preferences.hostname || !preferences.hostname[0])
	strncpy (preferences.hostname, DEFAULT_IRCSERVER, 255);
      if (!preferences.gfxtheme || !preferences.gfxtheme[0])
	strncpy (preferences.gfxtheme, DEFAULT_GFXTHEME, 64);
      if (!preferences.sfxtheme[0])
	strncpy (preferences.sfxtheme, DEFAULT_SFXTHEME, 64);

      return;			/* only set the default */
    }

  fh = open (default_file (), O_RDONLY);
  if (fh != -1)
    {
      fstat (fh, &st);
      cfg = malloc (st.st_size);
      if (cfg)
	{
	  read (fh, cfg, st.st_size);

	  /* strings */
	  cfg_get_str (cfg, "nickname ", preferences.nickname);
	  cfg_get_str (cfg, "sfxdir ", preferences.sfxdir);
	  cfg_get_str (cfg, "sfxtheme ", preferences.sfxtheme);
	  cfg_get_str (cfg, "gfxdir ", preferences.gfxdir);
	  cfg_get_str (cfg, "gfxtheme ", preferences.gfxtheme);
	  cfg_get_str (cfg, "hostname ", preferences.hostname);
	  cfg_get_str (cfg, "playerid ", preferences.playerid);
	  preferences.port = cfg_get_int (cfg, "port ");
	  preferences.ignore_motd = cfg_get_int (cfg, "ignore_motd ");
	  preferences.saveonexit = cfg_get_int (cfg, "saveonexit ");
	  preferences.auto_decline = cfg_get_int (cfg, "auto_decline ");
	  preferences.observe_decline = cfg_get_int (cfg, "observe_decline ");

	  free (cfg);
	}
      close (fh);
    }
  else
    {
      /* put in default values */
      strcpy (preferences.nickname, username);
      strcpy (preferences.gfxdir, DEFAULT_GFXDIR);
      strcpy (preferences.gfxtheme, DEFAULT_GFXTHEME);
      strcpy (preferences.sfxtheme, DEFAULT_SFXTHEME);
      strcpy (preferences.sfxdir, DEFAULT_SFXDIR);
      strcpy (preferences.hostname, DEFAULT_IRCSERVER);
      preferences.port = 6667;
      preferences.saveonexit = 1;
    }
  if (!preferences.hostname || !preferences.hostname[0])
    strcpy (preferences.hostname, DEFAULT_IRCSERVER);
  if (!preferences.gfxtheme || !preferences.gfxtheme[0])
    strcpy (preferences.gfxtheme, DEFAULT_GFXTHEME);
}

int
load_theme_config (void)
{
  char *cfg, *file = malloc (512);
  struct stat st;
  int fh;

  memset (&preferences.theme_data, 0, sizeof (struct _themedata));

  GFXPATH ("theme.conf", file);
  fh = open (file, O_RDONLY);
  free (file);
  if (fh != -1)
    {
      fstat (fh, &st);
      cfg = malloc (st.st_size);
      if (cfg)
	{
	  read (fh, cfg, st.st_size);

	  /* strings */
	  cfg_get_str (cfg, "invite ", preferences.theme_data.invite);
	  cfg_get_str (cfg, "observe ", preferences.theme_data.observe);
	  cfg_get_str (cfg, "background ", preferences.theme_data.background);
	  cfg_get_str (cfg, "p1_small ", preferences.theme_data.p1_small);
	  cfg_get_str (cfg, "p1_last ", preferences.theme_data.p1_last);
	  cfg_get_str (cfg, "p1_stone ", preferences.theme_data.p1_stone);
	  cfg_get_str (cfg, "p2_small ", preferences.theme_data.p2_small);
	  cfg_get_str (cfg, "p2_last ", preferences.theme_data.p2_last);
	  cfg_get_str (cfg, "p2_stone ", preferences.theme_data.p2_stone);
	  cfg_get_str (cfg, "xruler ", preferences.theme_data.xruler);
	  cfg_get_str (cfg, "yruler ", preferences.theme_data.yruler);
	  cfg_get_str (cfg, "marker ", preferences.theme_data.marker);

	  preferences.theme_data.background_line_xoffset =
	    cfg_get_int (cfg, "background_line_xoffset ");
	  preferences.theme_data.background_line_yoffset =
	    cfg_get_int (cfg, "background_line_yoffset ");
	  preferences.theme_data.background_tile_size =
	    cfg_get_int (cfg, "background_tile_size ");
	  preferences.theme_data.stone_xoffset =
	    cfg_get_int (cfg, "stone_xoffset ");
	  preferences.theme_data.stone_yoffset =
	    cfg_get_int (cfg, "stone_yoffset ");

	  free (cfg);
	}
      close (fh);
      return 0;
    }
  return 1;
}

int
save_config (void)
{
  int fh;

  fh = open (default_file (), O_TRUNC | O_WRONLY | O_CREAT, 0600);
  if (fh != -1)
    {
      /* strings */
      cfg_put_str (fh, "version ", VERSION);
      cfg_put_str (fh, "nickname ", preferences.nickname);
      cfg_put_str (fh, "sfxdir ", preferences.sfxdir);
      cfg_put_str (fh, "sfxtheme ", preferences.sfxtheme);
      cfg_put_str (fh, "gfxdir ", preferences.gfxdir);
      cfg_put_str (fh, "gfxtheme ", preferences.gfxtheme);
      cfg_put_str (fh, "hostname ", preferences.hostname);
      cfg_put_str (fh, "playerid ", preferences.playerid);

      cfg_put_int (fh, preferences.port, "port ");
      cfg_put_int (fh, preferences.ignore_motd, "ignore_motd ");
      cfg_put_int (fh, preferences.saveonexit, "saveonexit ");
      cfg_put_int (fh, preferences.auto_decline, "auto_decline ");
      cfg_put_int (fh, preferences.observe_decline, "observe_decline ");

      close (fh);
      return 1;
    }
  return 0;
}
