/*
        Artificial intelligence for Squaroid
                by Yair Chuchem
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <time.h>

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#define VOID 0
#define BEST_INITIAL_VALUE ((board.turn > 0) ? -666 : 666)
#define WIN_VALUE ((board.turn > 0) ? 200 : -200)
#define LEVELS 1
#define WIN_SCORE 150
#define BOARD_SIZE 8
#define FOR_EACH_PLACE \
	for (check.x = 0; check.x < BOARD_SIZE; check.x++) \
		for (check.y = 0; check.y < BOARD_SIZE; check.y++) \
			if (board.stone[check.x][check.y] == VOID)

static struct LevelMap
{
  int move;
  int level;
} Levels[] = {
  { 2, 2 },
  { 5, 3 }, 
  { 13, 4}, 
  { 18, 5 },
  { 22, 6 },
  { 26, 7 },
  { 29, 8 },
  { 31, 9 },
  { 0, 0 }
};

typedef struct
{
  int points[2];		// 0 = blue   1 = red
  int turn;
  int win;			// 0 = not yet   1 = blue   -1 = red
  int stone[BOARD_SIZE][BOARD_SIZE];	// 0 = void   1 = blue   -1 = red
}
Board;

typedef struct
{
  int points[2];
  int turn;
  int win;
  int x, y;
}
Backup;

typedef struct
{
  int x, y, value;
}
Place;

Board board;
int score[BOARD_SIZE];
int ai_Level, ai_Level_Cur;
int ai_is_playing = 0;
int ai_move_num = 0;
static int move_value_iterations = 0;

void
checkLevelMap()
{
  int i;
  for (i = 0; Levels[i].move; i++)
    if (ai_move_num == Levels[i].move)
    {
      ai_Level_Cur = Levels[i].level;
#ifdef AI_DEBUG
      fprintf(stderr, "[AI] Setting level %d in move %d\n", ai_Level_Cur,
	  ai_move_num);
#endif
      break;
    }
}

void
score_init ()
{
  int blah;

  score[0] = 0;
  for (blah = 1; blah < BOARD_SIZE; blah++)
    score[blah] = (blah + 1) * (blah + 1);
}

int
board_stone (int x, int y)
{
  return (x >= 0 && y >= 0 && x < BOARD_SIZE
	  && y < BOARD_SIZE) ? board.stone[x][y] : 0;
}

void
move_do (int x, int y, Backup * backup)
{
  int cx, cy;
  int add = 0;

  if (backup)
    {
      backup->points[0] = board.points[0];
      backup->points[1] = board.points[1];
      backup->turn = board.turn;
      backup->x = x;
      backup->y = y;
    }
  board.stone[x][y] = board.turn;
  for (cx = 0; cx < BOARD_SIZE; cx++)
    for (cy = 0; cy < BOARD_SIZE; cy++)
      if (board.stone[cx][cy] == board.turn &&
	  ((board_stone (x + y - cy, y + cx - x) == board.turn &&
	    board_stone (cx + y - cy, cy + cx - x) == board.turn) ||
	   (board_stone (x + cy - y, y + x - cx) == board.turn &&
	    board_stone (cx + cy - y, cy + x - cx) == board.turn)))
	{
	  add += score[abs (cx - x) + abs (cy - y)];
	}
  board.points[(board.turn > 0) ? 0 : 1] += add / 2;
  if (board.points[(board.turn > 0) ? 0 : 1] >= WIN_SCORE)
    board.win = board.turn;
  else
    board.turn = -board.turn;
}

void
move_undo (Backup * backup)
{
  board.stone[backup->x][backup->y] = VOID;
  board.points[0] = backup->points[0];
  board.points[1] = backup->points[1];
  board.turn = backup->turn;
  board.win = 0;
}

int
move_value (int x, int y, int levels)
{
  Place check;
  int dis;
  int best_value;
  Backup backup;

  move_value_iterations++;
  move_do (x, y, &backup);
  if (board.win)
    {
      dis = WIN_VALUE;
      move_undo (&backup);
      return dis;
    }
  if (!(--levels))
    {
      dis = board.points[0] - board.points[1];
      move_undo (&backup);
      return dis;
    }
  best_value = BEST_INITIAL_VALUE;
  FOR_EACH_PLACE
  {
    check.value = move_value (check.x, check.y, levels);
    if ((board.turn == 1) ? (check.value > best_value)
	: (check.value < best_value))
      best_value = check.value;
  }
  move_undo (&backup);
  return best_value;
}

// return which point on the board we put on
int
ai_move ()
{
  Place check;
  Place best;
  static int ai_move_iterations;
  
  best.value = BEST_INITIAL_VALUE;
  ai_move_iterations = 0; move_value_iterations = 0;
  FOR_EACH_PLACE
  {
    ai_move_iterations ++;
#ifdef AI_DEBUG
    fprintf(stderr, "AI_MOVE_ITERATIONS: %d / move_value: %d\n", 
	ai_move_iterations, move_value_iterations);
#endif
    check.value = move_value (check.x, check.y, ai_Level_Cur);
    if (
	((board.
	  turn == 1) ? (check.value > best.value) : (check.value <
						     best.value))
	|| (check.value == best.value && random () % 30 == 0))
      memcpy (&best, &check, sizeof (Place));
  }
  if (ai_Level > 3)
    checkLevelMap();
  else
    ai_Level_Cur = ai_Level;
  ai_move_num++;

  //fprintf(stderr, "MOVE %c%d\n", best.x+'A', '8'-best.y-'0');
  return best.x + best.y * BOARD_SIZE;
}

void
board_init ()
{
  struct timeval tv1, tv2;

#if 0
  if (!gettimeofday(&tv1, 0))
  {
    long speed = 0;
    int i;
    fprintf(stderr, "START: %ld.%ld\n", tv1.tv_sec, tv1.tv_usec);
    for ( i = 0; i < 100000; i++)
      { }
    gettimeofday(&tv2, 0);
    fprintf(stderr, "STOP: %ld.%ld\n", tv2.tv_sec, tv2.tv_usec);
    if (tv2.tv_sec == tv1.tv_sec)
      speed = (tv2.tv_usec - tv1.tv_usec);
    fprintf(stderr, "SPEED: %ld\n", speed);
    
  }
#endif
  
  srandom (time (NULL));
  memset (&board, 0, sizeof (Board));
  board.turn = 1;
  ai_move_num = 1;
  if (ai_Level < 1)
    ai_Level = 1;
  ai_Level_Cur = 1;
  ai_is_playing = 1;
}

