/* Squaroid
 * Copyright (C) 1999,2000 Lukas Schrder
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "squaroid.h"

#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <gdk/gdkprivate.h>


extern void move_do(int, int, void *);

void board_draw_background(Game *game)
{
  gint x, y;
  GdkPixmap *s;
  GdkBitmap *b;
  char buffer[512];

  GFXPATH(preferences.theme_data.xruler, buffer);
  if (gdk_imlib_load_file_to_pixmap(buffer, &s, &b) && !game->xruler)
  {
    game->xruler = gtk_pixmap_new(s, b);
    gdk_imlib_free_pixmap(s);
  }
  GFXPATH(preferences.theme_data.yruler, buffer);
  if (gdk_imlib_load_file_to_pixmap(buffer, &s, &b) && !game->yruler)
  {
    game->yruler = gtk_pixmap_new(s, b);
    gdk_imlib_free_pixmap(s);
  }
  
  GFXPATH(preferences.theme_data.background,buffer);
  if (gdk_imlib_load_file_to_pixmap(buffer, &s, &b) && !game->bg_tile)
    {
      game->bg_tile = gtk_pixmap_new(s, b);
      gdk_imlib_free_pixmap(s);
    }

 
  for (y = 0; y < 8; y++)
    for (x = 0; x < 8; x++)
      gdk_draw_pixmap(game->buffer, game->gc,
	  GTK_PIXMAP(game->bg_tile)->pixmap,
	  0, 0, x*BGTILE_SIZE+RULER_OFFSET, y*BGTILE_SIZE+RULER_OFFSET, 
	  -1, -1);
	
  gdk_draw_pixmap(game->buffer, game->gc,
      GTK_PIXMAP(game->xruler)->pixmap, 0, 0, RULER_OFFSET, 0, -1, -1);
  gdk_draw_pixmap(game->buffer, game->gc,
      GTK_PIXMAP(game->xruler)->pixmap, 0, 0, RULER_OFFSET, 8*50+RULER_OFFSET, -1, -1);
  gdk_draw_pixmap(game->buffer, game->gc,
      GTK_PIXMAP(game->yruler)->pixmap, 0, 0, 0, RULER_OFFSET, -1, -1);
  gdk_draw_pixmap(game->buffer, game->gc,
      GTK_PIXMAP(game->yruler)->pixmap, 0, 0, 8*50+RULER_OFFSET, RULER_OFFSET,
      -1, -1);

   /* if it's an observation, we have to also paint the moves already made */
  if (game->mode == GAME_OBSERVE)
  {
    int tmp_move = game->current_move;
    enum _player tmp_player = game->current_player;
    game->p1_score = 0;
    game->p2_score = 0;
    for (x = 0; x < tmp_move; x++)
      {
	game->current_move = x;
	game->current_player = game->moves[x].owner;
	game_paint_stone (game, game->moves[x], ADD_STONE);
	game_find_squares (game);
      }
    game->current_player = tmp_player;
    game->current_move = tmp_move;
  }
  
  gdk_draw_pixmap(game->canvas->window, game->gc, game->buffer,
                  0, 0, 0, 0, -1, -1);
}

void
game_canvas_put_stone (Game * game, gint xf, gint yf)
{
  if (xf > 7 || yf > 7		/* check for legal values */
      || game->game_over	/* and an ongoing game */
      || game->points[xf][yf] != NONE)
    {
      return;
    }

  if (game->mode == GAME_AI)
    move_do (xf, yf, 0);

  game_paint_old_lines (game);
  game_paint_old_stones (game);

  game->points[xf][yf] = game->current_player;
  switch (game->current_player)
    {
    case PLAYER1:
      if (GTK_PIXMAP (game->spot_blue_last)->mask)
	{
	  gdk_gc_set_clip_mask (game->gc,
				GTK_PIXMAP (game->spot_blue_last)->mask);
	  gdk_gc_set_clip_origin (game->gc,
				  xf * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
				  yf * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET);
	}
      gdk_draw_pixmap (game->buffer, game->gc,
		       GTK_PIXMAP (game->spot_blue_last)->pixmap,
		       0, 0,
		       xf * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
		       yf * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET, -1, -1);
      break;
    case PLAYER2:
      if (GTK_PIXMAP (game->spot_red_last)->mask)
	{
	  gdk_gc_set_clip_mask (game->gc,
				GTK_PIXMAP (game->spot_red_last)->mask);
	  gdk_gc_set_clip_origin (game->gc,
				  xf * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
				  yf * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET);
	}
      gdk_draw_pixmap (game->buffer, game->gc,
		       GTK_PIXMAP (game->spot_red_last)->pixmap,
		       0, 0,
		       xf * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
		       yf * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET, -1, -1);
    default:
      break;
    };
  gdk_gc_set_clip_mask (game->gc, NULL);
  gdk_draw_pixmap (game->canvas->window, game->gc, game->buffer,
		   xf * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
		   yf * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET,
		   xf * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
		   yf * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET,
		   BGTILE_SIZE, BGTILE_SIZE);

  game->moves[game->current_move].owner = game->current_player;
  game->moves[game->current_move].c.x = xf + 1;
  game->moves[game->current_move].c.y = yf + 1;

  game_canvas_update_last_stone (game);
  game_find_squares (game);

  if (!game->made_square)
    sq_play_sound (SND_PUTSTONE);

  game_next_move (game);
}

void
game_canvas_put_marker (Game * game, gint xf, gint yf, int mode)
{
  GtkWidget *pixmap;
  if (xf > 7 || yf > 7 || xf < 0 || yf < 0)	/* check for legal values */
    return;

  if (mode == ADD_STONE)
    {
      pixmap = game->bg_marker;
      game->marker[xf][yf] = 1;
    }
  else
    {
      pixmap = game->bg_tile;
      game->marker[xf][yf] = NONE;
    }

  if (GTK_PIXMAP (pixmap)->mask)
    {
      gdk_gc_set_clip_mask (game->gc, GTK_PIXMAP (pixmap)->mask);
      gdk_gc_set_clip_origin (game->gc, 
	  xf * BGTILE_SIZE + RULER_OFFSET, 
	  yf * BGTILE_SIZE + RULER_OFFSET);
    }

  gdk_draw_pixmap (game->buffer, game->gc, GTK_PIXMAP (pixmap)->pixmap,
		   0, 0, 
		   xf * BGTILE_SIZE + RULER_OFFSET, 
		   yf * BGTILE_SIZE + RULER_OFFSET, -1, -1);

  gdk_gc_set_clip_mask (game->gc, NULL);
  gdk_draw_pixmap (game->canvas->window, game->gc, game->buffer,
		   xf * BGTILE_SIZE + RULER_OFFSET, 
		   yf * BGTILE_SIZE + RULER_OFFSET,
		   xf * BGTILE_SIZE + RULER_OFFSET, 
		   yf * BGTILE_SIZE + RULER_OFFSET,
		   BGTILE_SIZE, BGTILE_SIZE);
}

void
game_canvas_update_last_stone (Game * game)
{
  if (game && game->current_move > 0)
    game_paint_stone (game, game->moves[game->current_move - 1], ADD_STONE);
}

void
game_paint_stone (Game * game, struct movedata move, int mode)
{
  switch (mode)
    {
    case ADD_STONE:
      if (move.owner == PLAYER1)
	{
	  if (GTK_PIXMAP (game->spot_blue)->mask)
	    {
	      gdk_gc_set_clip_mask (game->gc,
				    GTK_PIXMAP (game->spot_blue)->mask);
	      gdk_gc_set_clip_origin (game->gc,
	        (move.c.x - 1) * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
		(move.c.y - 1) * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET);
	    }
	  gdk_draw_pixmap (game->buffer, game->gc,
	   GTK_PIXMAP (game->spot_blue)->pixmap, 0, 0,
	   (move.c.x - 1) * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
	   (move.c.y - 1) * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET, -1, -1);
	}
      else
	{
	  if (GTK_PIXMAP (game->spot_red)->mask)
	    {
	      gdk_gc_set_clip_mask (game->gc,
				    GTK_PIXMAP (game->spot_red)->mask);
	      gdk_gc_set_clip_origin (game->gc,
		(move.c.x - 1) * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
	        (move.c.y - 1) * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET);
	    }
	  gdk_draw_pixmap (game->buffer, game->gc,
	    GTK_PIXMAP (game->spot_red)->pixmap, 0, 0,
	    (move.c.x - 1) * BGTILE_SIZE + ST_XOFFSET + RULER_OFFSET,
	    (move.c.y - 1) * BGTILE_SIZE + ST_YOFFSET + RULER_OFFSET, -1, -1);
	}
      break;
    case DEL_STONE:
      break;
    default:
      break;
    }

  gdk_gc_set_clip_mask (game->gc, NULL);
  gdk_draw_pixmap (game->canvas->window, game->gc, game->buffer,
		   (move.c.x - 1) * BGTILE_SIZE + RULER_OFFSET,
		   (move.c.y - 1) * BGTILE_SIZE + RULER_OFFSET,
		   (move.c.x - 1) * BGTILE_SIZE + RULER_OFFSET,
		   (move.c.y - 1) * BGTILE_SIZE + RULER_OFFSET, -1, -1);
}

void
game_paint_square (Game * game, struct point p[], int mode)
{
  switch (mode)
    {
    case NEW_SQUARE:
      if (game->current_player == PLAYER1)
	gdk_gc_set_foreground (game->gc, &game->col_blue);
      else
	gdk_gc_set_foreground (game->gc, &game->col_red);
      break;
    case OLD_SQUARE:
      if (game->current_player == PLAYER1)
	gdk_gc_set_foreground (game->gc, &game->col_red_after);
      else
	gdk_gc_set_foreground (game->gc, &game->col_blue_after);
      break;
    default:
      break;
    }

  gdk_draw_line (game->buffer, game->gc,
		 (p[0].x - 1) * BGTILE_SIZE + BGTILE_XOFFSET + RULER_OFFSET,
		 (p[0].y - 1) * BGTILE_SIZE + BGTILE_YOFFSET + RULER_OFFSET, 
		 (p[1].x - 1) * BGTILE_SIZE + BGTILE_XOFFSET + RULER_OFFSET,
		 (p[1].y - 1) * BGTILE_SIZE + BGTILE_YOFFSET + RULER_OFFSET);
  gdk_draw_line (game->buffer, game->gc,
		 (p[0].x - 1) * BGTILE_SIZE + BGTILE_XOFFSET + RULER_OFFSET,
		 (p[0].y - 1) * BGTILE_SIZE + BGTILE_YOFFSET + RULER_OFFSET,
		 (p[2].x - 1) * BGTILE_SIZE + BGTILE_XOFFSET + RULER_OFFSET,
		 (p[2].y - 1) * BGTILE_SIZE + BGTILE_YOFFSET + RULER_OFFSET);
  gdk_draw_line (game->buffer, game->gc,
		 (p[1].x - 1) * BGTILE_SIZE + BGTILE_XOFFSET + RULER_OFFSET,
		 (p[1].y - 1) * BGTILE_SIZE + BGTILE_YOFFSET + RULER_OFFSET,
		 (p[3].x - 1) * BGTILE_SIZE + BGTILE_XOFFSET + RULER_OFFSET,
		 (p[3].y - 1) * BGTILE_SIZE + BGTILE_YOFFSET + RULER_OFFSET);
  gdk_draw_line (game->buffer, game->gc,
		 (p[2].x - 1) * BGTILE_SIZE + BGTILE_XOFFSET + RULER_OFFSET,
		 (p[2].y - 1) * BGTILE_SIZE + BGTILE_YOFFSET + RULER_OFFSET,
		 (p[3].x - 1) * BGTILE_SIZE + BGTILE_XOFFSET + RULER_OFFSET,
		 (p[3].y - 1) * BGTILE_SIZE + BGTILE_YOFFSET + RULER_OFFSET);

}

void
game_paint_old_lines (Game * game)
{
  struct square *the_square;
  GSList *list = game->old_lines;

  while (list)
    {
      the_square = (struct square *) list->data;

      game_paint_square (game, the_square->corners, OLD_SQUARE);

      list = list->next;
    }
  sq_cleanup_simple_glist (game->old_lines);
  game->old_lines = 0;
}

void
game_paint_old_stones (Game * game)
{
  struct movedata *old_stone;
  GSList *list = game->old_stones;

  while (list)
    {
      old_stone = (struct movedata *) list;

      game_paint_stone (game, *old_stone, ADD_STONE);

      list = list->next;
    }
  sq_cleanup_simple_glist (game->old_stones);
  game->old_stones = 0;
}

/* CALLBACKS */

void cb_canvas_realize(GtkWidget *w)
{
  gint width, height;
  GdkGCPrivate *gcpriv;
  XGCValues xvalues;
  Game *game = find_game_by_canvas(w);
	
  if (!game)
		return;
  gdk_window_get_size(game->canvas->window, &width, &height);
  game->buffer = gdk_pixmap_new(game->canvas->window, width, height, -1);
  game->gc = gdk_gc_new(game->canvas->window);

  gcpriv = (GdkGCPrivate*)game->gc;
  xvalues.line_width = 3;
  XChangeGC(gcpriv->xdisplay, gcpriv->xgc, GCLineWidth, &xvalues);

  board_draw_background(game);
}

gint
cb_canvas_expose (GtkWidget * w, GdkEventExpose * e)
{
  Game *game = find_game_by_canvas (w);

  if (!game)
    return FALSE;
  gdk_draw_pixmap (game->canvas->window, game->gc, game->buffer,
		   e->area.x, e->area.y, e->area.x, e->area.y,
		   e->area.width, e->area.height);

  return TRUE;
}

gboolean
cb_canvas_button_press (GtkWidget * w, GdkEventButton * e, gpointer data)
{
  gint xf, yf;
  Game *game = (Game *) data;

  if (!game)
    return FALSE;
  xf = (gint) floor ((e->x - RULER_OFFSET) / BGTILE_SIZE);
  yf = (gint) floor ((e->y - RULER_OFFSET) / BGTILE_SIZE);

  /*
     if ((game->point_pressed.c.x != xf && game->point_pressed.c.x != 9)
     || (game->point_pressed.c.y != yf && game->point_pressed.c.y != 9))
     game_paint_stone(game, game->point_pressed, DEL_STONE);
   */

  /* set some data where the button was pressed to check for when releasing */
  switch (e->button)
    {
    case 1:			/* left button */
      game->point_pressed.c.x = xf;
      game->point_pressed.c.y = yf;
      break;
    case 2:			/* middle button */
      break;
    case 3:			/* right button */
      game->marker_pressed.c.x = xf;
      game->marker_pressed.c.y = yf;
      break;
    }
  /*  game_paint_stone(game, game->point_pressed, ADD_STONE); */

  return TRUE;
}

gboolean
cb_canvas_button_release (GtkWidget * w, GdkEventButton * e, gpointer data)
{
  gint xf, yf;
  char buf[MAX_MESSAGE_SIZE];
  Game *game = (Game *) data;

  if (!game)
    return FALSE;
  xf = (gint) floor ( (e->x - RULER_OFFSET) / BGTILE_SIZE);
  yf = (gint) floor ( (e->y - RULER_OFFSET) / BGTILE_SIZE);

  if (e->button == 1)
    {
      if (game->point_pressed.c.x != xf || game->point_pressed.c.y != yf)
	{
	  /*    game_paint_stone(game, game->point_pressed, DEL_STONE); */
	  return FALSE;
	}
      game->point_pressed.c.x = 9;
      game->point_pressed.c.y = 9;

      if ((game->mode == GAME_NETWORK || game->mode == GAME_TOURNAMENT)
	  && game->who_am_i != game->current_player)
	return FALSE;
      if (xf > 7 || yf > 7 || game->points[xf][yf] != NONE)
	return FALSE;

      if (game->mode == GAME_NETWORK || game->mode == GAME_TOURNAMENT)
	{
	  sqirc_send (game->connection,
		      "NOTICE %s :MOVE %c%d %d %d\r\n",
		      game->opponent, 'A' + xf, 7 - yf + 1,
		      game->movelimit.time_left, game->movelimit.time_left_o);
	  snprintf (buf, MAX_MESSAGE_SIZE, "MOVE %c%d", 'A' + xf, 7 - yf + 1);
	  sq_game_observer_notify (game, "NOTICE", buf);
	}

      /* cleanup field before putting the stone! */
      if (game->marker[xf][yf] != NONE)
	game_canvas_put_marker (game, xf, yf, DEL_STONE);

      game_canvas_put_stone (game, xf, yf);
    }
  else if (e->button == 3)
    {
      if (game->marker_pressed.c.x != xf || game->marker_pressed.c.y != yf)
	return FALSE;
      game->marker_pressed.c.x = 9;
      game->point_pressed.c.y = 9;

      if (game->points[xf][yf] != NONE)
	return FALSE;

      if (game->marker[xf][yf] == NONE)
	game_canvas_put_marker (game, xf, yf, ADD_STONE);
      else
	game_canvas_put_marker (game, xf, yf, DEL_STONE);
    }

  return TRUE;
}
