/* Squaroid
 * Copyright (C) 1999,2000 Lukas Schrder
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "squaroid.h"

static Game *game_invited = 0;

#define IS_INVITATION_CHANGE (game->invited != NOT_INVITED)

void cb_playrequest_answer_yes(GtkWidget *, gpointer);
void cb_playrequest_answer_no(GtkWidget *, gpointer);
gint cb_playrequest_answer_delete(GtkWidget *, gpointer);

void cb_playrequest_abort(GtkWidget *, gpointer);
void cb_playrequest_invite(GtkWidget *, gpointer);
void cb_playrequest_firstplayer(GtkWidget *, gpointer);
void cb_playrequest_gamemode(GtkWidget *, gpointer);
void cb_playrequest_timelimit(GtkWidget *, gpointer);

static void cb_dialog_focus_in(GtkWidget *, GdkEventFocus *, gpointer);

/* this function now handles 3 states:
	 
	 1. game->invited == NOT_INVITED
	 #  first time invitation request
	 
	 2. game->invited == BEEN_INVITED
	 #  we received an invitation or a change of values
	 
	 3. game->invited == HAVE_INVITED
 	 # we sent out the first time invitation request,
	 # but the opponent changed something
*/
void game_play_request(Game *game)
{
  GtkWidget *ok, *abort, *button, *vbox, *frame, *frame_box;
	GtkWidget *notebook;
	GSList *plugins = 0;
	
	/* only one invitation request can be made at once */
	timeout_del_pending_play_request(game);	
	if (game_invited == game) 
		{
			chatroom_chatmessage(CHATROOM(game->connection),
													 STARTOFF " %s just tried to invite you, \n"
													 "    but you already have an invite dialog to "
													 "this player open\n"
													 STARTOFF " The invitation has been answered, "
													 "\n    telling %s that you are busy inviting her\n",
													 game->e_opponent->nickname,
													 game->e_opponent->nickname);
			sqirc_send(game->connection, "NOTICE %s :INVITE_BUSY\r\n",
								 game->opponent);
			return;
		}
	game_invited = game;
	
	game->invite_data = malloc(sizeof(InvitationData));
	bzero(game->invite_data, sizeof(InvitationData));
  game->dialog = gtk_dialog_new();
	gtk_signal_connect(GTK_OBJECT(game->dialog), "focus_in_event",
										 cb_dialog_focus_in, game);
	
	if (game->invited == NOT_INVITED || game->invited == HAVE_INVITED)
		gtk_window_set_title(GTK_WINDOW(game->dialog), "Invite Opponent");
	else if (game->invited == BEEN_INVITED)
		gtk_window_set_title(GTK_WINDOW(game->dialog), "You've been invited");
	
  gtk_window_set_position(GTK_WINDOW(game->dialog), GTK_WIN_POS_CENTER);
  gtk_container_border_width(GTK_CONTAINER(game->dialog), 5);
	
	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(game->dialog)->vbox),
										 notebook, TRUE, TRUE, 0);
	
	vbox = gtk_vbox_new(FALSE, 5);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
													 vbox, gtk_label_new("default"));
	
	/* create frame to hold 'Game Modus' */
	frame = gtk_frame_new("Game Variation");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 5);
	frame_box = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(frame), frame_box);
	
	button = 0;
	button = gtk_radio_button_new_with_label(NULL,"Standard");
	gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 cb_playrequest_gamemode, NULL);
	/* add an option for each plugin the opponent has loaded too! */
	plugins = sq_plugin_list();
	while (plugins)
		{
			Plugin *plugin = (Plugin *)plugins->data;
			GtkWidget *p_page = 0;             /* plugin's notebook Page */
			
			if ( !sq_player_has_plugin(game->e_opponent, plugin->callbacks->name) )
				{
 					plugins = plugins->next;
					continue;
				}
			button = gtk_radio_button_new_with_label(RGROUP(button),
																							 plugin->callbacks->name);
			gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
			gtk_signal_connect(GTK_OBJECT(button), "toggled",
												 cb_playrequest_gamemode,
												 (void *)plugin);
			
			if (plugin->callbacks->OnInvite != 0)
				{
					if (plugin->callbacks->OnInvite(game, &p_page) == PLUGIN_SUCCESS)
						gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
																		 p_page,
																		 gtk_label_new(plugin->callbacks->name));
				}
			
			if (GAME_PLUGIN(game) == plugin)
				ACTIVATE(button);
			
			plugins = plugins->next;
		}
	
	/* create frame to hold 'First Move'-Options */
  frame = gtk_frame_new("Player to make first move");
  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 5);
  frame_box = gtk_vbox_new(FALSE,5);
  gtk_container_add(GTK_CONTAINER(frame), frame_box);
	
	/* create first-move option: "You" */
	button = 0;
	if (game->invited == HAVE_INVITED || game->invited == NOT_INVITED)
		button = gtk_radio_button_new_with_label(NULL, game->e_me->nickname);
	else
		button = gtk_radio_button_new_with_label(NULL, game->e_opponent->nickname);
	
  gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 cb_playrequest_firstplayer, (void*)YOU_FIRST);
	
	if ( IS_INVITATION_CHANGE
			 && game->options.first_player == YOU_FIRST)
		ACTIVATE(button);
	
	/* create first-move option: "Opponent" */  
	if (game->invited == HAVE_INVITED || game->invited == NOT_INVITED)
		button = gtk_radio_button_new_with_label(RGROUP(button),
																						 game->e_opponent->nickname); 
	else
		button = gtk_radio_button_new_with_label(RGROUP(button),
																						 game->e_me->nickname);
  gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 cb_playrequest_firstplayer, (void*)OPPONENT_FIRST);
	if ( IS_INVITATION_CHANGE
			 && game->options.first_player == OPPONENT_FIRST)
		ACTIVATE(button);
	
	/* next option: "random" */
	button = gtk_radio_button_new_with_label(RGROUP(button), "Random");
	gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 cb_playrequest_firstplayer, (void*)RANDOM_FIRST);
	if ( IS_INVITATION_CHANGE
			 && game->options.first_player == RANDOM_FIRST)
		ACTIVATE(button);
	
	/* time limitation box */
	button = 0;
	game->invite_data->unit_label = gtk_label_new("Unit: Minutes");
	game->invite_data->adjustment = (GtkAdjustment*)gtk_adjustment_new(0, 0, 1, 1, 0, 0);
	
	/*
		hbox = gtk_hbox_new(FALSE,5);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
	*/
	
	frame = gtk_frame_new("Timed Game");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 5);
	frame_box = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(frame), frame_box);
	
	button = gtk_radio_button_new_with_label(NULL, "Off");
	gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	if ( IS_INVITATION_CHANGE
			 && game->options.timeout == LIMIT_OFF )
		ACTIVATE(button);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 cb_playrequest_timelimit, (void*)LIMIT_OFF);
	
	button = gtk_radio_button_new_with_label(RGROUP(button),"On, time per Game");
	gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 cb_playrequest_timelimit, (void*)LIMIT_PERGAME);
	if ( IS_INVITATION_CHANGE
			 && game->options.timeout == LIMIT_PERGAME )
		{
			gtk_label_set_text(GTK_LABEL(game->invite_data->unit_label),
												 "Unit: Minutes");
			ACTIVATE(button);
		}
	
	button = gtk_radio_button_new_with_label(RGROUP(button),
																					 "On, time per Move");
	gtk_box_pack_start(GTK_BOX(frame_box), button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "toggled",
										 cb_playrequest_timelimit, (void*)LIMIT_PERMOVE);
	if ( IS_INVITATION_CHANGE
			 && game->options.timeout == LIMIT_PERMOVE )
		{
			gtk_label_set_text(GTK_LABEL(game->invite_data->unit_label),
												 "Unit: Seconds");
			ACTIVATE(button);
		}
	
	game->invite_data->adjustment->value = game->movelimit.time_max;
	game->invite_data->hscale = gtk_hscale_new(game->invite_data->adjustment);
	gtk_scale_set_digits(GTK_SCALE(game->invite_data->hscale), 0);
	gtk_box_pack_start(GTK_BOX(frame_box), game->invite_data->hscale,
										 FALSE, FALSE, 0);
	
  gtk_box_pack_start(GTK_BOX(frame_box), game->invite_data->unit_label,
										 TRUE, TRUE, 0);
	
	
	/*
		frame = gtk_frame_new("Timed Game Settings");
		gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 5);
		frame_box = gtk_vbox_new(FALSE, 5);
		gtk_container_add(GTK_CONTAINER(frame), frame_box);
		
		text_custom_limit = gtk_entry_new_with_max_length(4);
		gtk_box_pack_start(GTK_BOX(frame_box), text_custom_limit, FALSE, FALSE, 0);
		gtk_entry_set_text(GTK_ENTRY(text_custom_limit), "300");
		if (IS_INVITATION_CHANGE
		&& game->options.timeout == LIMIT_CUSTOM )
		{
		ACTIVATE(button);
		chatroom_debugmessage(" INVITATION_CHANGE time_max=[%d]\n",
		game->movelimit.time_max);
		sq_entry_set_text(GTK_ENTRY(text_custom_limit), "%d",
		game->movelimit.time_max);
		}
	*/
	
  /* OK button: Invite */
	if (IS_INVITATION_CHANGE)
		ok = gtk_button_new_with_label("Send changes");
	else 
		ok = gtk_button_new_with_label("Invite");
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(game->dialog)->action_area),
										 ok, TRUE, TRUE, 0);
	
	/* give the button's life a sense: connect the signals */
  gtk_signal_connect(GTK_OBJECT(ok), "clicked",
										 cb_playrequest_invite, game);
	
	if (IS_INVITATION_CHANGE)
		{
			ok = gtk_button_new_with_label("Accept");
			gtk_signal_connect(GTK_OBJECT(ok), "clicked",
												 cb_playrequest_answer_yes, (gpointer)game);
			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(game->dialog)->action_area),
												 ok, TRUE, TRUE, 0);
		}
	
	/* Cancel button: Abort */
  abort = gtk_button_new_with_label("Abort");
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(game->dialog)->action_area),
										 abort, TRUE, TRUE, 0);
	
	if (!IS_INVITATION_CHANGE)
		gtk_signal_connect(GTK_OBJECT(abort), "clicked",
											 cb_playrequest_abort, game);
	else
		gtk_signal_connect(GTK_OBJECT(abort), "clicked",
											 cb_playrequest_answer_no, (gpointer)game);
	
	/* and now let the world know about this */
  gtk_widget_show_all(GTK_DIALOG(game->dialog)->vbox);
  gtk_widget_show_all(GTK_DIALOG(game->dialog)->action_area);
	
	/* first view or limit is off */
	if (!IS_INVITATION_CHANGE || game->options.timeout <= LIMIT_OFF)
		{
			gtk_widget_hide(game->invite_data->hscale);
			gtk_widget_hide(game->invite_data->unit_label);
		}
	
	if (game->pending_invite)
		game->invited = HAVE_INVITED;
	game->pending_invite = 0;
	
  gtk_widget_show(game->dialog);
  game->dialog_shown = TRUE;
}

void game_play_request_abort(Game *game)
{
	if (game == game_invited)
		game_invited = 0;
}

/* cb for the invite dialog: called when the radio buttons states change */
void cb_playrequest_firstplayer(GtkWidget *w, gpointer data)
{
  game_invited->options.first_player = (int)data;
}

void cb_playrequest_gamemode(GtkWidget *w, gpointer data)
{
	game_invited->options.plugin = data;
}

void cb_playrequest_timelimit(GtkWidget *w, gpointer data)
{
	Game *game = game_invited;
	game->options.timeout = (guint)data;
	
	if ((guint)data == LIMIT_OFF)
		{
			if (game->invite_data->hscale
					&& GTK_WIDGET_VISIBLE(game->invite_data->hscale))
				gtk_widget_hide(game->invite_data->hscale);
			if (game->invite_data->unit_label
					&& GTK_WIDGET_VISIBLE(game->invite_data->unit_label))
				gtk_widget_hide(game->invite_data->unit_label);
		}
	else if ( (guint)data == LIMIT_PERGAME )
		{
			game->invite_data->adjustment->value = 5.0;
			game->invite_data->adjustment->lower = 2.0;
			game->invite_data->adjustment->upper = 60.0;
			game->invite_data->adjustment->step_increment = 1.0;
			game->invite_data->adjustment->page_increment = 5.0;
			game->invite_data->adjustment->page_size = 0.0;
			
			gtk_label_set_text(GTK_LABEL(game->invite_data->unit_label),
												 "Unit: Minutes");
			gtk_adjustment_changed(game->invite_data->adjustment);
			if (game->invite_data->hscale
					&& GTK_WIDGET_VISIBLE(game->invite_data->hscale) == FALSE)
				gtk_widget_show(game->invite_data->hscale);
			if (game->invite_data->unit_label
					&& GTK_WIDGET_VISIBLE(game->invite_data->unit_label) == FALSE)
				gtk_widget_show(game->invite_data->unit_label);
			
		}
	else if ( (guint)data == LIMIT_PERMOVE )
		{
			game->invite_data->adjustment->value = 60.0;
			game->invite_data->adjustment->lower = 15.0;
			game->invite_data->adjustment->upper = 180.0;
			game->invite_data->adjustment->step_increment = 5.0;
			game->invite_data->adjustment->page_increment = 15.0;
			game->invite_data->adjustment->page_size = 0.0;
			
			gtk_label_set_text(GTK_LABEL(game->invite_data->unit_label),
												 "Unit: Seconds");
			gtk_adjustment_changed(game->invite_data->adjustment);
			if (game->invite_data->hscale
					&& GTK_WIDGET_VISIBLE(game->invite_data->hscale) == FALSE)
				gtk_widget_show(game->invite_data->hscale);
			if (game->invite_data->unit_label
					&& GTK_WIDGET_VISIBLE(game->invite_data->unit_label) == FALSE)
				gtk_widget_show(game->invite_data->unit_label);
		}
}

void cb_playrequest_answer_yes(GtkWidget *w, gpointer data)
{
  Game *game = (Game *)data;
	int i = 0;
  
  if (!game)
		return;
	
	game_invited = 0;
	
	srand(time(0));
	i = 1+(int)(10.0*rand()/(RAND_MAX+1.0));
	if (game->options.first_player == RANDOM_FIRST)
		game->options.first_player = (i>4)?YOU_FIRST:OPPONENT_FIRST;
	
	sqirc_send(game->connection,
						 "NOTICE %s :INVITE_ACK %d\r\n",
						 game->opponent,
						 game->options.first_player);
	
  game_start_playing(game, GAME_NETWORK);
  
  gtk_widget_destroy(game->dialog);
  game->dialog = 0;
  game->dialog_shown = FALSE;
  gtk_widget_set_sensitive(top_window, TRUE);
}

void cb_playrequest_answer_no(GtkWidget *w, gpointer data)
{
  Game *game = (Game *)data;

  if (!game) 
		return;
	game_invited = 0;
  sqirc_send(game->connection,
						 "NOTICE %s :INVITE_REJ\r\n", game->opponent);
	
  while(gtk_events_pending())
    gtk_main_iteration();
	
  gtk_widget_set_sensitive(top_window, TRUE);
  game_cleanup(game);
}

gint cb_playrequest_answer_delete(GtkWidget *w, gpointer data)
{
	game_invited = 0;
  return TRUE;
}

void cb_playrequest_abort(GtkWidget *w, gpointer data)
{
  game_cleanup((Game *)data);
	game_invited = 0;
}

void cb_playrequest_invite(GtkWidget *w, gpointer data)
{
  Game *game = (Game *)data;
	
	if (game->options.timeout != LIMIT_OFF)
		{
			/*int value = atoi(gtk_entry_get_text(GTK_ENTRY(text_custom_limit)));*/
			int value = (int)game->invite_data->adjustment->value;
			
			game->movelimit.time_max = game->movelimit.time_left = value;
			chatroom_debugmessage(" custom move limit is %d\n",
														game->movelimit.time_max);
		}
	
  sqirc_send(game->connection,
						 "NOTICE %s :INVITE %d %d %d %d %s\r\n",
						 game->opponent,
						 game->options.first_player,
						 game->options.timeout,
						 game->movelimit.time_max,
						 game->e_opponent->lag,
						 (!GAME_PLUGIN(game))?"-":GAME_PLUGIN(game)->callbacks->name);
	
	/* associate plugindata with game, as plugin->data gets overwritten soon */	
	if ( GAME_PLUGIN(game) )
		game->plugindata = GAME_PLUGIN(game)->data;
	if (GAME_PLUGIN(game) && GAME_PLUGIN(game)->callbacks->OnInviteSend)
		{
			GAME_PLUGIN(game)->callbacks->OnInviteSend(game);
			sqirc_send(game->connection, "NOTICE %s :INVITE_END\r\n",
								 game->opponent); 
		}
	
  gtk_widget_destroy(game->dialog);
  game->dialog_shown = 0;
  game->dialog = 0;
	game_invited = 0;
	game->invite_data->hscale = 0;
	game->invite_data->adjustment = 0;
}

void cb_dialog_focus_in(GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
	game_invited = (Game *)data; /* find_game_by_dialog(widget); */
	if (game_invited == 0)
		fprintf(stderr, "DialogFocusIn:  FATAL could not find game for "
						"this dialog!\n");
}
