/* Squaroid
 * Copyright (C) 1999 Lukas Schrder
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "squaroid.h"

void cb_observe_select_row(GtkWidget *w, gint row, gint col,
													 GdkEventButton *e, Game *game);
void cb_observe_send(GtkWidget *w, Game *game);

void game_observe_request(Game *game)
{
  game->pending_observe = 0;
  
  if (g_slist_length(game->e_opponent->opponents) > 0) 
		{
			GtkWidget *opponent_list, *button;
			GSList *list;
			time_t start = time(0);
			
			while(game->dialog_shown && start > time(0)-10)
				gtk_main_iteration();
			
			game->dialog = gtk_dialog_new();
			gtk_window_set_title(GTK_WINDOW(game->dialog),
													 "Choose the opponent to observe");
			gtk_window_set_position(GTK_WINDOW(game->dialog), GTK_WIN_POS_CENTER);
			gtk_container_set_border_width(GTK_CONTAINER(game->dialog), 10);
			gtk_widget_set_usize(game->dialog, 250, 150);
			
			opponent_list = gtk_clist_new(1);
			gtk_clist_freeze(GTK_CLIST(opponent_list));
			list = game->e_opponent->opponents;
			while (list)
				{
					gtk_clist_append(GTK_CLIST(opponent_list), (char **)&list->data);
					list = list->next;
				}
			gtk_clist_thaw(GTK_CLIST(opponent_list));
			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(game->dialog)->vbox),
												 opponent_list, TRUE, TRUE, 2);
			gtk_signal_connect(GTK_OBJECT(opponent_list), "select-row",
												 cb_observe_select_row, (gpointer)game);
	
			button = gtk_button_new_with_label("Observe");
			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(game->dialog)->action_area),
												 button, TRUE, FALSE, 2);
			gtk_signal_connect(GTK_OBJECT(button), "clicked",
												 cb_observe_send, (gpointer)game);

			button = gtk_button_new_with_label("Cancel");
			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(game->dialog)->action_area),
												 button, TRUE, FALSE, 2);
			gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
																gtk_widget_destroy, (gpointer)game->dialog);
			gtk_signal_connect_object(GTK_OBJECT(game->dialog), "destroy",
																(GtkSignalFunc)game_cleanup,
																(gpointer)game);
			
			gtk_widget_show_all(GTK_DIALOG(game->dialog)->vbox);
			gtk_widget_show_all(GTK_DIALOG(game->dialog)->action_area);
			
			game->dialog_shown = TRUE;
			gtk_widget_show(game->dialog);
		}
}

/* game_encode_board_data() encodes the current field into printable characters
 * to be transmitted through IRC. The other option would have been DCC
 * but i find it cooler this way, because encoding/decoding makes
 * save games possible
 *
 * For each stone on the Board this function sets the bits of move_data
 * Bits 8 7 6 are for the x location (0..7)
 * Bits 5 4 3 are for the y location (0..7)
 * Bit    2   is  for the owner of the field (0 for PLAYER1 / 1 for PLAYER2)
 * Bit    1   is  for the current player of the game; flags same as for Bit 2  
 * each resulting byte is then split into two, highest 4 bits and lowest 4 bits
 * each of the two fields value is added to the base 'A' enforcing it to be
 * a printable, allowed character
 */
char *game_encode_board_data(Game *game)
{
  int i = 0, k = 0, j = 0;
  int data_size = game->current_move*sizeof(char);
  unsigned char move_data;
  /* +2 for the first bytes containing who_am_i and currentplayer */
  char *enc_md = malloc(data_size*2+2+1); 
  memset(enc_md, 0, sizeof(enc_md));

  enc_md[j++] = '0' + game->current_player;
  enc_md[j++] = '0' + game->who_am_i;

  for(i = 0, k = 0; i < game->current_move; i++, k++)
    {
      move_data = 
				(((game->moves[i].c.x-1) << 5) | ((game->moves[i].c.y-1) << 2)
				 | (((game->moves[i].owner == PLAYER1) & 0x01)<<1)
				 | (((game->current_player == PLAYER1) & 0x01)<<0)
				 );
      enc_md[j++] = 'A' + ((move_data & 0xf0) >> 4);
      enc_md[j++] = 'A' + (move_data & 0x0f);
    }
  enc_md[j++] = '\0';
  return enc_md;
}

/* want to guess? yep, this function decodes the string that encode produced */
void game_decode_board_data(Game *game, char *raw_data)
{
  int i = 0, k = 0;
  unsigned char move_data;
  char *dec_md = raw_data+2;
	
  game->current_player = *raw_data - '0';
  game->who_am_i = *(raw_data+1) - '0';
	
  if (strlen(dec_md))
		for (i = 0, k = 0; i < game->current_move*2; i++, k++)
			{
				move_data = ((dec_md[i] - 'A') << 4) | (dec_md[++i] - 'A');
				game->moves[k].c.x   = ((move_data & 0xe0) >> 5) +1;
				game->moves[k].c.y   = ((move_data & 0x1c) >> 2) +1;
				game->moves[k].owner = ((move_data & 0x02) ? PLAYER1 : PLAYER2);
				game->current_player = ((move_data & 0x01) ? PLAYER1 : PLAYER2);
			}

}

void cb_observe_send(GtkWidget *w, Game *game)
{
  sqirc_send(game->connection,
						 "NOTICE %s :OBSERVE %s\r\n",
						 game->opponent, game->observed);
	
  gtk_signal_disconnect_by_func(GTK_OBJECT(game->dialog),
																(GtkSignalFunc)game_cleanup,
																(gpointer)game);
  
  game->dialog_shown = FALSE;
  gtk_widget_destroy(game->dialog);
	game->dialog = 0;
}

void cb_observe_select_row(GtkWidget *w, gint row, gint col,
													 GdkEventButton *e, Game *game)
{
  char *buffer;
  gtk_clist_get_text(GTK_CLIST(w), row, col, &buffer);

	bzero(game->observed, BUFSIZE);
  strncpy(game->observed, buffer, BUFSIZE);
}
