/* Squaroid
 * Copyright (C) 1999,2000 Lukas Schrder
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "squaroid.h"

#define DISTANCE(A,B) (B.x-A.x)*(B.x-A.x)+(B.y-A.y)*(B.y-A.y)

void game_find_squares(Game *game)
{
  struct point player1_points[32];
  struct point player2_points[32];
  int i;
  int p1_c = 0;
  int p2_c = 0;
  for (i=0;i<=game->current_move;i++)
		{
			switch(game->moves[i].owner)
				{
				case PLAYER1:
					player1_points[p1_c].x = game->moves[i].c.x;
					player1_points[p1_c++].y = game->moves[i].c.y;
					break;
				case PLAYER2:
					player2_points[p2_c].x = game->moves[i].c.x;
					player2_points[p2_c++].y = game->moves[i].c.y;
					break;
				default:
					break;
				}
		}
  if (game->current_player == PLAYER1)
		{
			if (p1_c >= 4)
				game_check_for_squares(game, player1_points,p1_c);
		} 
	else
		{
			if (p2_c >= 4)
				game_check_for_squares(game, player2_points,p2_c);
		}
}

void game_check_for_squares(Game *game, struct point points[],
														int n_points)
{
  int h, i, j, k, p, s = 0;
  h = n_points-1;
  for (i = 0; i < h; i++)
		for (j=i+1; j < h; j++)
			for (k=j+1; k < h; k++)
				{
					s +=
						p = game_is_square(game, points[h],points[i],points[j],points[k]);
					if (p)
						game_add_to_score(game, p);
				}
	
	game->made_square = s?TRUE:FALSE;
	sq_play_square_sound(s);
	
  /* redraw the whole window */
  gdk_draw_pixmap(game->canvas->window, game->gc, game->buffer,
									0, 0, 0, 0, -1, -1);
}

int game_is_square(Game *game, struct point pp1, struct point pp2,
                                     struct point pp3, struct point pp4)
{
  int d1,d2,d3,d4,d5,d6;
  struct point p[4];
  struct square *the_square;
  struct movedata *move;
  p[0] = pp1;
  p[1] = pp2;
  p[2] = pp3;
  p[3] = pp4;
  game_sort_points(p,4);
  d1 = DISTANCE(p[0],p[1]);
  d2 = DISTANCE(p[0],p[2]);
  d3 = DISTANCE(p[1],p[3]);
  d4 = DISTANCE(p[2],p[3]);
  d5 = DISTANCE(p[0],p[3]);
  d6 = DISTANCE(p[1],p[2]);

  if (d1 == d2 && d1 == d3 && d1 == d4 && d5 == d6)
		{
			game_paint_square(game, p, NEW_SQUARE);
			
			the_square = malloc(sizeof(struct square));
			bzero(the_square, sizeof(struct square));
			memcpy(the_square->corners, &p, sizeof(struct square));
			ADD_LIST_ITEM(game->old_lines, the_square);
			
			move = malloc(sizeof(struct movedata));
			move->owner = game->current_player;
			move->c.x = p[0].x;
			move->c.y = p[0].y;
			ADD_LIST_ITEM(game->old_stones, move);
			
			move = malloc(sizeof(struct movedata));
			move->owner = game->current_player;
			move->c.x = p[1].x;
			move->c.y = p[1].y;
			ADD_LIST_ITEM(game->old_stones, move);
			
			move = malloc(sizeof(struct movedata));
			move->owner = game->current_player;
			move->c.x = p[2].x;
			move->c.y = p[2].y;
			ADD_LIST_ITEM(game->old_stones, move);
			
			move = malloc(sizeof(struct movedata));
			move->owner = game->current_player;
			move->c.x = p[3].x;
			move->c.y = p[3].y;
			ADD_LIST_ITEM(game->old_stones, move);
			
			if (GAME_PLUGIN(game) == 0
					|| GAME_PLUGIN(game)->callbacks->OnSquare == 0)
				return (p[3].x-p[0].x+1)*(p[3].x-p[0].x+1);
			else
				return GAME_PLUGIN(game)->callbacks->OnSquare(game, p[3].x-p[0].x+1);
		}
  return 0;
}

/* selection sort */
void game_sort_points(struct point points[],int n_points)
{
  struct point t;
  int i, j, min;
  for (i=0; i<n_points;i++)
    {
      min = i;
      for (j = i+1; j < n_points; j++)
		if (points[j].x < points[min].x
			|| ( (points[j].y < points[min].y)
				 && (points[j].x == points[min].x))
			) min = j;
	  
      t = points[min];
      points[min] = points[i];
      points[i] = t;
    }
}
