/* Squaroid
 * Copyright (C) 1999,2000 Lukas Schrder
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "squaroid.h"

/* game_ai.c */
extern int ai_Level;
extern int ai_is_playing;

static GtkWidget *dialog;
static GtkWidget *pref_gfxdir;
static GtkWidget *pref_sfxdir;
static GtkWidget *pref_gfxtheme;
static GtkWidget *pref_sfxtheme;
static GtkWidget *dirhandler_target;
static GtkWidget *themehandler_target;
static GtkWidget *aigame_custom_level;
static int init = 1;

void menucallback_help (GtkWidget *, gpointer);
void menucallback_about (GtkWidget *, gpointer);
void menucallback_savedefaults (GtkWidget *, gpointer);
void menucallback_saveonexit (GtkWidget *, gpointer);
void menucallback_skipmotd (GtkWidget *, gpointer);
void menucallback_auto_decline (GtkWidget *, gpointer);
void menucallback_observe_decline (GtkWidget *, gpointer);
void menucallback_start_sologame (GtkWidget *, gpointer);
void menucallback_start_aigame (GtkWidget *, gpointer);
void menucallback_preferences (GtkWidget *, gpointer);
void menucallback_unconnect (GtkWidget *, gpointer);

static GtkItemFactoryEntry menu_items[] = { {"/_Squaroid", NULL,
					     NULL, 0, "<Branch>"},
{"/Squaroid/_Connect to server", "<control>C",
 menucallback_connect, 0, NULL},
{"/Squaroid/_Disconnect from server", "<control>D",
 menucallback_unconnect, 0, NULL},
{"/Squaroid/start solo _Game", "<control>G",
 menucallback_start_sologame, 0, NULL},
{"/Squaroid/start _Ai game", "<control>A",
 menucallback_start_aigame, 0, NULL},
{"/Squaroid/sep1", NULL,
 NULL, 0, "<Separator>"},
{"/Squaroid/_Save settings now", "<control>S",
 menucallback_savedefaults, 0, NULL},
{"/Squaroid/sep2", NULL,
 NULL, 0, "<Separator>"},
{"/Squaroid/_Quit...", "<control>Q",
 callback_destroy, 0, NULL},
{"/Options", NULL,
 NULL, 0, "<Branch>"},
{"/Options/_Preferences", "<control>P",
 menucallback_preferences, 0, NULL},
{"/Options/sep1", NULL,
 NULL, 0, "<Separator>"},
{"/Options/auto decline _Invitations", "<control>I",
 menucallback_auto_decline, 0, "<ToggleItem>"},
{"/Options/auto decline _Observers", "<control>O",
 menucallback_observe_decline, 0, "<ToggleItem>"},
{"/Options/skip _Motd", "<control>M",
 menucallback_skipmotd, 0, "<ToggleItem>"},
{"/Options/sep2", NULL,
 NULL, 0, "<Separator>"},
{"/Options/save settings on exit", NULL,
 menucallback_saveonexit, 0, "<ToggleItem>"},
{"/Help", NULL,
 NULL, 0, "<LastBranch>"},
{"/Help/about squaroid", NULL,
 menucallback_about, 0, NULL,}
};

GtkWidget *
createmenus ()
{
  int nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  GtkWidget *menu_item;

  accel_group = gtk_accel_group_new ();

  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
     or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
     the accelerator table while generating menus.
   */
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
				       accel_group);

  /* Attach the new accelerator group to the window. */
  gtk_accel_group_attach (accel_group, GTK_OBJECT (top_window));

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

#define CHECK_MENU(_path, _value) \
        { menu_item = gtk_item_factory_get_widget(item_factory,_path); \
        gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(menu_item), \
                                      _value); }
  init = 1;

  if (preferences.auto_decline)
    CHECK_MENU ("/Options/auto decline Invitations", preferences.auto_decline)
      if (preferences.observe_decline)
      CHECK_MENU ("/Options/auto decline Observers",
		  preferences.observe_decline) if (preferences.ignore_motd)
	CHECK_MENU ("/Options/skip Motd",
		    preferences.ignore_motd) if (preferences.saveonexit)
	  CHECK_MENU ("/Options/save settings on exit",
		      preferences.saveonexit) init = 0;

  /* Finally, return the actual menu bar created by the item factory. */
  return gtk_item_factory_get_widget (item_factory, "<main>");
}

static char *help_commands = "
  Squaroid Help
  == == == == == == =
  Available chatroom commands
  * /exit, /quit, /part
  Disconnect from server, and reopen the Connect Dialog
  * /disconnect
  Disconnect from server, and DO NOT reopen the Connect Dialog
  * /score < nickname >
  Get the scores for player called < nickname >
  */allscores
  Get scores for all players currently on this channel.
  * /solo Start a solo game.This is one of three ways to start a solo game;)
  * /msg < nickname > <message > |/notice < nickname > <message >
    Send a private < message > to < nickname >
    The score commands only work if the SquaroidHost bot
    is also logged in with the nickname `Squaroid'.";

static char *help_options = "
  Available Options
  * Skip MOTD
  Skip the Message Of The Day that each IRC server sends
  you.
  * Auto Decline
  This automatically declines every game challenge you
  might receive.I.e.if you are playing a game and
do
    not
    want to be disturbed this option is for you
    * Save Settings on Exit
    I think this is self explanatory ! ";  
    void menucallback_help (GtkWidget * w, gpointer data)
    {
      chatroom_chatmessage (NULL, help_commands);
      chatroom_chatmessage (NULL, help_options);
    }

void menucallback_about (GtkWidget * w, gpointer data)
{
  char buf[4096];
  snprintf (buf, sizeof (buf),
	    "\nSquaroid\n"
	    "Homepage               : http://www.azzit.de/squaroid/\n"
	    "Author                 : Lukas Schrder <lukas@azzit.de>\n"
	    "Version                : %s\n"
	    "Compiled               : %s %s\n"
	    "Current Sounddir       : %s\n"
	    "Current Graphicsdir    : %s\n"
	    "Current Theme          : %s\n",
	    VERSION, __DATE__, __TIME__,
	    preferences.sfxdir, preferences.gfxdir, preferences.gfxtheme);
  sq_print_on_current_chatwindow ("%s", buf);
}

void menucallback_savedefaults (GtkWidget * w, gpointer data)
{
  save_config ();
}

void menucallback_saveonexit (GtkWidget * w, gpointer data)
{
  if (init)
    return;
  preferences.saveonexit = !preferences.saveonexit;
}

void menucallback_skipmotd (GtkWidget * w, gpointer data)
{
  if (init)
    return;
  preferences.ignore_motd = !preferences.ignore_motd;
}

void menucallback_auto_decline (GtkWidget * w, gpointer data)
{
  if (init)
    return;
  preferences.auto_decline = !preferences.auto_decline;
}

void menucallback_observe_decline (GtkWidget * w, gpointer data)
{
  if (init)
    return;
  preferences.observe_decline = !preferences.observe_decline;
}

void menucallback_start_sologame (GtkWidget * w, gpointer data)
{
  Game *game = game_create (CHATROOM_DUMMY);

  strcpy (game->name, "You");
  strcpy (game->opponent, "You");

  game_start_playing (game, GAME_SOLO);
}

#define BOX_PACK(_box_, _widget_) gtk_box_pack_start(GTK_BOX(_box_), _widget_, TRUE, TRUE, 2)
#define RADIO_GROUP(_button_) gtk_radio_button_group(GTK_RADIO_BUTTON(_button_))
#define ACTIVATE(_but) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_but), TRUE)
#define RADIO_ON_TOGGLE(_button_, _data_) gtk_signal_connect(GTK_OBJECT(_button_), "toggled", cb_aigame_toggle, (gpointer)_data_)

void cb_aigame_toggle (GtkWidget * w, gpointer data)
{
  ai_Level = (int) data;
}

void cb_aigame_playertoggle (GtkWidget *w, gpointer data)
{
  Game *game = (Game *)data;
  static int playertoggle_count = 0;
  if (!game)
  {
    playertoggle_count = 0;     /* HACK! */
    return;
  }
  playertoggle_count++;
  if (playertoggle_count % 2)
    game->options.first_player = 0;
  else
    game->options.first_player = YOU_FIRST;
}

void cb_start_aigame (GtkWidget * w, gpointer data)
{
  if (ai_Level > 3)
  {
    char *text;
    text = gtk_entry_get_text(GTK_ENTRY(aigame_custom_level));
    if (!text || !text[0])
      ai_Level = 4; 
    else
      ai_Level = atoi(text);
    
  }
  game_start_playing ((Game *) data, GAME_AI);
  gtk_widget_destroy (((Game *) data)->dialog);
}

void menucallback_start_aigame (GtkWidget * w, gpointer data)
{
  Game *game;
  GtkWidget *dialog, *button, *frame, *box, *text, *hbox;

  if (ai_is_playing)
    {
      sq_print_on_current_chatwindow (STARTON " You are already playing an "
				      "AI Game\n");
      return;
    }

  game = game_create (CHATROOM_DUMMY);
  game->invited = HAVE_INVITED;
  strcpy (game->name, "Human");
  strcpy (game->opponent, "CPU");

  dialog = game->dialog = gtk_dialog_new ();
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
  frame = gtk_frame_new ("Choose the AI Level");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), frame, TRUE, TRUE,
		      2);
  gtk_container_border_width (GTK_CONTAINER (frame), 10);
  box = gtk_vbox_new (TRUE, TRUE);
  gtk_container_add (GTK_CONTAINER (frame), box);

  button = gtk_radio_button_new_with_label (NULL, "Novice");
  BOX_PACK (box, button);
  RADIO_ON_TOGGLE (button, 1);
  if (ai_Level <= 1)
    ACTIVATE (button);

  button = gtk_radio_button_new_with_label (RADIO_GROUP (button), "Advanced");
  BOX_PACK (box, button);
  RADIO_ON_TOGGLE (button, 2);
  if (ai_Level == 2)
    ACTIVATE (button);

  button = gtk_radio_button_new_with_label (RADIO_GROUP (button), "Expert");
  BOX_PACK (box, button);
  RADIO_ON_TOGGLE (button, 3);
  if (ai_Level >= 3)
    ACTIVATE (button);

  button = gtk_radio_button_new_with_label (RADIO_GROUP (button), "Custom");
  BOX_PACK (box, button);
  RADIO_ON_TOGGLE (button, 4);
  aigame_custom_level = button = gtk_entry_new ();
  BOX_PACK (box, button);
  sq_entry_set_text(GTK_ENTRY(button), "%d", 10);

  button = gtk_check_button_new_with_label("CPU goes first");
  BOX_PACK (box, button);
  cb_aigame_playertoggle(0, 0);  /* reset the counter */
  gtk_signal_connect(GTK_OBJECT(button), "toggled", cb_aigame_playertoggle,
      (gpointer)game);

  button = gtk_button_new_with_label ("Start");
  BOX_PACK (GTK_DIALOG (dialog)->action_area, button);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (cb_start_aigame), game);

  gtk_widget_show_all (dialog);

}

void dirselect_handler (gchar * path)
{
  gtk_entry_set_text (GTK_ENTRY (dirhandler_target), path);
}

void callback_preferences_browse (GtkWidget * w, gpointer data)
{
  GtkWidget *entry = (GtkWidget *) data;
  GtkWidget *browser;
  char *text;

  dirhandler_target = entry;
  text = gtk_entry_get_text (GTK_ENTRY (entry));
  browser = xmms_create_dir_browser ("Select a Directory",
				     text,
				     GTK_SELECTION_SINGLE, dirselect_handler);
  gtk_window_set_position (GTK_WINDOW (browser), GTK_WIN_POS_CENTER);
  gtk_widget_show (browser);
}

void themeselect_handler (gchar * theme)
{
  gtk_entry_set_text (GTK_ENTRY (themehandler_target), theme);
}

void callback_preferences_theme (GtkWidget * w, gpointer data)
{
  GtkWidget *entry = (GtkWidget *) data;
  GtkWidget *browser;
  char *text;

  if (entry == pref_sfxdir)
    themehandler_target = pref_sfxtheme;
  else if (entry == pref_gfxdir)
    themehandler_target = pref_gfxtheme;

  text = gtk_entry_get_text (GTK_ENTRY (entry));
  browser = sq_create_dir_browser_rel ("Select your Theme",
				       text,
				       GTK_SELECTION_SINGLE,
				       themeselect_handler);
  gtk_window_set_position (GTK_WINDOW (browser), GTK_WIN_POS_CENTER);
  gtk_widget_show (browser);
}

void callback_preferences_save (GtkWidget * w, gpointer data)
{
  char *oldtheme = g_strdup (preferences.gfxtheme);
  char *oldgfxdir = g_strdup (preferences.gfxdir);

  snprintf (preferences.gfxtheme, sizeof (preferences.gfxtheme),
	    gtk_entry_get_text (GTK_ENTRY (pref_gfxtheme)));
  snprintf (preferences.gfxdir, sizeof (preferences.gfxdir),
	    gtk_entry_get_text (GTK_ENTRY (pref_gfxdir)));

  if (!sq_check_files ())
    {
      g_free (oldtheme);
      g_free (oldgfxdir);
      gtk_widget_destroy (dialog);
    }
  else
    {
      snprintf (preferences.gfxtheme, sizeof (preferences.gfxtheme),
		oldtheme);
      snprintf (preferences.gfxdir, sizeof (preferences.gfxdir), oldgfxdir);
      g_free (oldtheme);
      g_free (oldgfxdir);
      chatroom_chatmessage (NULL,
			    STARTON STARTOFF
			    " Files are missing\nCheck your preferences "
			    STARTOFF STARTON "\n");
    }
}

void menucallback_preferences (GtkWidget * w, gpointer data)
{
  GtkWidget *frame, *label, *button, *table;

  dialog = gtk_dialog_new ();
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_title (GTK_WINDOW (dialog), "Squaroid Preferences");
  gtk_container_set_border_width (GTK_CONTAINER (dialog), 10);
  gtk_widget_set_usize (dialog, 500, 200);

  /* frame of the graphics preferences */
  frame = gtk_frame_new ("Graphics Preferences");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      frame, FALSE, FALSE, 0);

  table = gtk_table_new (2, 4, TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (table), 5);
  gtk_container_add (GTK_CONTAINER (frame), table);

  label = gtk_label_new ("Graphics Dir");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);

  pref_gfxdir = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (pref_gfxdir), preferences.gfxdir);
  gtk_table_attach (GTK_TABLE (table), pref_gfxdir, 1, 3, 0, 1,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);

  button = gtk_button_new_with_label ("Browse");
  gtk_table_attach (GTK_TABLE (table), button, 3, 4, 0, 1,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND, 5, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      callback_preferences_browse, pref_gfxdir);

  label = gtk_label_new ("Theme");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

  pref_gfxtheme = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (pref_gfxtheme), preferences.gfxtheme);
  gtk_table_attach (GTK_TABLE (table), pref_gfxtheme, 1, 3, 1, 2,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

  button = gtk_button_new_with_label ("Select");
  gtk_table_attach (GTK_TABLE (table), button, 3, 4, 1, 2,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND, 5, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      callback_preferences_theme, pref_gfxdir);
  /* END OF FRAME OF THE GRAPHICS PREFERENCES */

  /* frame of the sound preferences */
  frame = gtk_frame_new ("Sound Preferences");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      frame, FALSE, FALSE, 0);

  table = gtk_table_new (2, 4, TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (table), 5);
  gtk_container_add (GTK_CONTAINER (frame), table);

  label = gtk_label_new ("Sound Dir");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);

  pref_sfxdir = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (pref_sfxdir), preferences.sfxdir);
  gtk_table_attach (GTK_TABLE (table), pref_sfxdir, 1, 3, 0, 1,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);

  button = gtk_button_new_with_label ("Browse");
  gtk_table_attach (GTK_TABLE (table), button, 3, 4, 0, 1,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND, 5, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      callback_preferences_browse, pref_sfxdir);

  label = gtk_label_new ("Theme");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

  pref_sfxtheme = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (pref_sfxtheme), preferences.sfxtheme);
  gtk_table_attach (GTK_TABLE (table), pref_sfxtheme, 1, 3, 1, 2,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

  button = gtk_button_new_with_label ("Select");
  gtk_table_attach (GTK_TABLE (table), button, 3, 4, 1, 2,
		    GTK_EXPAND | GTK_FILL, GTK_EXPAND, 5, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      callback_preferences_theme, pref_sfxdir);
  /* END OF FRAME OF THE SOUND PREFERENCES */

  /* ############################################################### */
  button = gtk_button_new_with_label ("Save");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
		      TRUE, TRUE, 35);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (callback_preferences_save), 0);

  button = gtk_button_new_with_label ("Cancel");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
		      TRUE, TRUE, 35);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) dialog);

  gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
  gtk_widget_show_all (GTK_DIALOG (dialog)->action_area);

  gtk_widget_show (dialog);
}

void menucallback_connect (GtkWidget * w, gpointer data)
{
  show_dialog_connect ();
}

void menucallback_unconnect (GtkWidget * w, gpointer data)
{
  Chatroom *chatroom;

  if ((chatroom = find_chatroom_by_tablabel (current_tablabel)) == 0)
    {
      sq_print_on_current_chatwindow (STARTON " Change to the Chatwindow"
				      " you want to close the connection of\n");
      return;
    }

  chatroom_destroy (chatroom);
}
