/* Squaroid
 * Copyright (C) 1999,2000 Lukas Schrder
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "squaroid.h"

static SQIRC_FUNC_CMDSERVER(cmd_server_chatmsg);
static SQIRC_FUNC_CMDSERVER(cmd_server_motd);
static SQIRC_FUNC_CMDSERVER(cmd_server_nicknameinuse);
static SQIRC_FUNC_CMDSERVER(cmd_server_namreply);
static SQIRC_FUNC_CMDSERVER(cmd_server_whois);
struct server_command server_commands[] =
{ 
	{ 1, cmd_server_chatmsg },
	CMDIGNORE(2),
	CMDIGNORE(3),
	CMDIGNORE(4),
	CMDIGNORE(437),
	CMDIGNORE(ERR_CHANNELISFULL),
	CMDIGNORE(ERR_INVITEONLYCHAN),
	CMDIGNORE(ERR_BANNEDFROMCHAN),
	CMDIGNORE(ERR_BADCHANNELKEY),
	CMDIGNORE(RPL_LUSERCLIENT),
	CMDIGNORE(RPL_LUSERME),
	CMDIGNORE(RPL_LUSERCHANNELS),
	CMDIGNORE(RPL_UNKNOWN_265),
	CMDIGNORE(RPL_UNKNOWN_266),
	CMDIGNORE(RPL_ENDOFWHO),
	CMDIGNORE(333),
	CMDIGNORE(RPL_TOPIC),
	CMDIGNORE(329),
	CMDIGNORE(RPL_CHANNELMODEIS),
	CMDIGNORE(RPL_LISTEND),
	CMDIGNORE(RPL_AWAY),
	CMDIGNORE(RPL_ISON),
	CMDIGNORE(RPL_WHOISSERVER),
	CMDIGNORE(RPL_WHOWASUSER),
	CMDIGNORE(RPL_WHOISIDLE),
	CMDIGNORE(RPL_ENDOFWHOIS),
	CMDIGNORE(RPL_WHOISOPERATOR),
	CMDIGNORE(RPL_WHOISCHANNELS),
	CMDIGNORE(RPL_LISTSTART),
	CMDIGNORE(RPL_LIST),
	
	{ RPL_WHOISUSER, cmd_server_whois },
	{ RPL_WHOREPLY, cmd_server_whois },
	
	{ RPL_NAMREPLY, cmd_server_namreply },
	{ RPL_ENDOFNAMES, cmd_server_namreply },
	
	{ RPL_MOTDSTART, cmd_server_motd },
	{ RPL_MOTD, cmd_server_motd },
	{ RPL_ENDOFMOTD, cmd_server_motd },
	{ ERR_NOMOTD, cmd_server_motd },
	
	{ ERR_NICKNAMEINUSE, cmd_server_nicknameinuse },
	
	{ 0, NULL }
};

static SQIRC_FUNC_CMDCLIENT(cmd_client_error);
static SQIRC_FUNC_CMDCLIENT(cmd_client_notice);
static SQIRC_FUNC_CMDCLIENT(cmd_client_part);
static SQIRC_FUNC_CMDCLIENT(cmd_client_ping);
static SQIRC_FUNC_CMDCLIENT(cmd_client_privmsg);
static SQIRC_FUNC_CMDCLIENT(cmd_client_quit);
static SQIRC_FUNC_CMDCLIENT(cmd_client_join);
struct client_command client_commands[] =
{	{ "NOTICE",        cmd_client_notice  },
	{ "PRIVMSG",       cmd_client_privmsg },
	{ "PING",          cmd_client_ping    },
	{ "QUIT",          cmd_client_quit    },
	{ "PART",          cmd_client_part    },
	{ "ERROR",         cmd_client_error   },
	{ "JOIN",          cmd_client_join    },
	LAST_COMMAND
};

static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_updateplayer);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_invite);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviteend);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviteack);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviterej);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_gamequit);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_gametimeout);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_timeoutadjust);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_move);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviteagainack);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviteagainrej);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observe);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observeack);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observerej);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observeadd);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observereset);
static SQIRC_FUNC_CMDCLIENT(cmd_squaroid_playerid);
struct client_command squaroid_commands[] =
{ { "UPDATEPLAYER",     cmd_squaroid_updateplayer },
	{ "INVITE",           cmd_squaroid_invite },
	{ "INVITE_BUSY",      cmd_squaroid_invite },
	{ "INVITE_END",       cmd_squaroid_inviteend },
	{ "INVITE_ACK",       cmd_squaroid_inviteack },
	{ "INVITE_REJ",       cmd_squaroid_inviterej },
	{ "GAMEQUIT",         cmd_squaroid_gamequit },
	{ "GAMETIMEOUT",      cmd_squaroid_gametimeout },
	{ "TIMEOUTADJUST",    cmd_squaroid_timeoutadjust },
	{ "MOVE",             cmd_squaroid_move },
	{ "INVITE_AGAIN_ACK", cmd_squaroid_inviteagainack },
	{ "INVITE_AGAIN_REJ", cmd_squaroid_inviteagainrej },
	{ "OBSERVE",          cmd_squaroid_observe },
	{ "OBSERVE_ACK",      cmd_squaroid_observeack },
	{ "OBSERVE_REJ",      cmd_squaroid_observerej },
	{ "OBSERVE_ADD",      cmd_squaroid_observeadd },
	{ "OBSERVE_RESET",    cmd_squaroid_observereset },
	{ "PLAYERID",         cmd_squaroid_playerid },
	LAST_COMMAND
};

static SQIRC_FUNC_CMDCLIENT(cmd_ctcp_version);
static SQIRC_FUNC_CMDCLIENT(cmd_ctcp_opponents);
static SQIRC_FUNC_CMDCLIENT(cmd_ctcp_opponentsstart);
static SQIRC_FUNC_CMDCLIENT(cmd_ctcp_opponentsend);
static SQIRC_FUNC_CMDCLIENT(cmd_ctcp_opponentsentry);
static SQIRC_FUNC_CMDCLIENT(cmd_ctcp_action);
static SQIRC_FUNC_CMDCLIENT(cmd_ctcp_plugins);
struct client_command ctcp_commands[] =
{ { "VERSION",            cmd_ctcp_version },
	{ "OPPONENTS",          cmd_ctcp_opponents },
	{ "OPPONENTS_START",    cmd_ctcp_opponentsstart },
	{ "OPPONENTS_END",      cmd_ctcp_opponentsend },
	{ "OPPONENTS_ENTRY",    cmd_ctcp_opponentsentry },
	{ "ACTION",             cmd_ctcp_action },
	{ "PLUGINS",            cmd_ctcp_plugins },
	LAST_COMMAND
};

/* ############################################################### */

struct connection *connect_to_server()
{
	int ret = 0;
	struct irc_userdata userdata;
	struct connection *connection = 0;
	ulong flags = 0;

#if NET_DEBUG
	flags |= SQIRC_DEBUG;
#endif
	
  sq_status_set("Trying to connect to %s", preferences.hostname);
	snprintf(userdata.nickname, IRC_BUFNICK, "%s", preferences.nickname);
	snprintf(userdata.username, IRC_BUFUSER, "%s", g_get_user_name());
	snprintf(userdata.desc,     IRC_BUFSIZE,       "Squaroid Client");
	if ( (ret = sqirc_connect(preferences.hostname,
														preferences.port,
														flags,
														&userdata,
														&connection)) > IRCERR_BASE )
		{
			switch(ret)
				{
				case IRCERR_INVALID_HOSTNAME:
					fprintf(stderr, "**** Invalid hostname ***\n");
					chatroom_chatmessage(CHATROOM(connection),
															 STARTON " Invalid hostname\n");
					sq_status_set("Invalid hostname!");
					break;
					
				case IRCERR_LOOKUP_FAIL:
					fprintf(stderr, "**** Failed looking up host ***\n");
					chatroom_chatmessage(CHATROOM(connection),
															 STARTON " Failed looking up host\n");
					sq_status_set("Failed to look up host");
					break;
					
				case IRCERR_NOSOCKET:
					fprintf(stderr, "%s\n", strerror(errno));
					sq_status_set("Error creating socket");
					break;
					
				default:
					break;
				}
			return 0;
		}
	else if (ret == IRC_CONNECTED)
		{
			Chatroom *chatroom = find_chatroom_by_tablabel(current_tablabel);
			
			if (chatroom == 0 || chatroom->connection != NULL)
				chatroom = find_unconnected_chatroom();
			
			connection->data = chatroom;
			chatroom->connection = connection;
			
			sq_status_set("Connected successfully");
			chatroom_chatmessage(CHATROOM(connection),
															 STARTON " Successfully connected. "
															 "Now logging in\n");
		}
	return connection;
}

void sq_close_connection(struct connection *connection)
{
  sq_player_cleanup_all(CHATROOM(connection));
  chatroom_player_clearlist(CHATROOM(connection));
	sqirc_close_connection(connection);
  chatroom_chatmessage(CHATROOM(connection),
											 STARTON " Connection closed\n");
  sq_status_set("Disconnected!");
}



/* ######################################## */
SQIRC_FUNC_CMDSERVER(cmd_server_chatmsg)
{
	chatroom_chatmessage(CHATROOM(connection), STARTON " %s\n", data->message+1);
}

SQIRC_FUNC_CMDSERVER(cmd_server_motd)
{
	
	if (reply == RPL_ENDOFMOTD || reply == ERR_NOMOTD)
		{
			/*			connection->motd_skipped = TRUE; */
			sqirc_send(connection, "JOIN #squaroid\r\n");
			chatroom_join(CHATROOM(connection), "#squaroid");
			return;
		}
	
	if (preferences.ignore_motd)
		return;
	
	if (reply == RPL_MOTDSTART)
		chatroom_chatmessage(CHATROOM(connection), "\n");

	chatroom_chatmessage(CHATROOM(connection),
											 STARTON " %s\n", data->message+1);	
}

SQIRC_FUNC_CMDSERVER(cmd_server_nicknameinuse)
{
	chatroom_chatmessage(CHATROOM(connection),
													 STARTON STARTOFF " Nickname \0033%s\003 already "
													 "in use " STARTOFF STARTON "\n",
													 preferences.nickname);
	CHATROOM(connection)->connection = 0;
	connection->data = 0;
	sqirc_close_connection(connection);
}

SQIRC_FUNC_CMDSERVER(cmd_server_namreply)
{
	int i;
	Chatroom *chatroom;

	if (reply == RPL_ENDOFNAMES)
		{
			chatroom_thaw(CHATROOM(connection));
			return;
		}

	chatroom_freeze(CHATROOM(connection));
	
	chatroom = find_chatroom_by_channel(connection, data->list[1]);
	/* start with i=2 to skip = and the channel name */
	for (i=2; data->list[i]; i++)
		{
			char *nick = data->list[i];
			
			/* throw away unwanted characters; TODO: find use for this info */
			while (*nick == ':' || *nick == '@' || *nick == '+' || *nick == '#')
				nick++;
			
			chatroom_player_add(chatroom, nick, 100);
		}
}

SQIRC_FUNC_CMDSERVER(cmd_server_whois)
{
	Chatroom *chatroom;
	Player *player;
	char *realname = data->message;
	char *nick;
	
	if (reply == RPL_WHOISUSER)
		nick = data->list[0];
	else
		nick = data->list[4];
	
	while (*realname != ':' && *realname != 0)
		realname++;
	
	if (reply != RPL_WHOISUSER)
		while (*realname != ' ' && *realname != 0)
			realname++;

	chatroom = CHATROOM(connection);
	if ( (player = find_player_by_nick(connection, nick)) != 0)
		{
			strncpy(player->realname, ++realname, 200);
			
			if (chatroom->gamehost[0] == 0
					&& strstr(player->realname, "Squaroid Host"))
				{
					strncpy(chatroom->gamehost, player->nickname, SMALLBUF);
					chatroom_chatmessage(chatroom,
															 STARTOFF " found Squaroid Host \0032%s\003\n",
															 player->nickname);
				}
			if ( strstr(player->realname, "Squaroid Client") == 0)
				{
					int row;
					row = chatroom_player_inlist(CHATROOM(connection), player->nickname);
					chatroom_player_mark_invalid(CHATROOM(connection), row-1);
				}
		}
	
}

/* ########################################################## */

/* :\001VERSION\001
	 request/answer for the version information */
SQIRC_FUNC_CMDCLIENT(cmd_ctcp_version)
{
	Game *game = 0;
	Player *player = 0;
	int answer = !strcmp("NOTICE", command);
	char *version = 0;
	int row = 0;
	time_t now = time(0);
	
	if (!answer)             /* privmsg: send out the version answer */
		{
			struct utsname un;
			player = find_player_by_nick(connection, preferences.nickname);
			
			if (uname (&un) < 0)
				return;
			
			sqirc_send(connection, 
								 "NOTICE %s :\001VERSION Squaroid " VERSION "\001\r\n",
								 data->from.nick);
			player->last_update = now;
			return;
		}
	
	/* handle the response */
	player = 0;
	if (!(player = find_player_by_nick(connection, data->from.nick)))
		{
			/* this should not happen!!! */
			player = sq_player_create(CHATROOM(connection));
			strcpy(player->nickname, data->from.nick);
			chatroom_debugmessage(" received a ctcpversion answer from an "
														"unknown player. this should not happen!\n");
		}
	
	player->flags |= PLAYER_HAVE_VERSION;
	version = data->list[2];
	player->major_version = atoi(version);
	player->minor_version = atoi(version+2);
	player->revision = atoi(version+5);
	player->valid = ((strcasecmp("squaroid",data->list[1]) == 0)
									 && (player->major_version >= VALID_MAJOR_VERSION)
									 && (player->minor_version >= VALID_MINOR_VERSION)
									 && (player->revision      >= VALID_REVISION));
	/*
		fprintf(stderr, "PLAYER major=%d  minor=%d revision=%d\n",
		player->major_version, player->minor_version, player->revision);
	*/
	
	if (player->last_update && player->lag < 120)
		player->lag = now - player->last_update;

	chatroom_debugmessage(" found a lag of %d [%ld:%ld] to player %s\n", 
												player->lag,
												now,
												player->last_update,
												player->nickname);
	player->last_update = now;
	
	if (!player->valid
			&& (row = chatroom_player_inlist(CHATROOM(connection),data->from.nick))) 
		{
			/* row-1 because in chatroom_player_inlist we add 1
				 to also return a true value if row == 0 */
			chatroom_player_mark_invalid(CHATROOM(connection), row-1);
		}
	
	game = find_game_by_opponent(connection, data->from.nick);
	if (!player->valid && game->timeout_play)
		{
			timeout_del_pending_play_request(game);
			sq_notify_wrong_version(game);
		}
	else if (player->valid && game->timeout_play
					 && (player->flags & PLAYER_HAVE_PLUGINS))
		{
			timeout_del_pending_play_request(game);
			game_play_request(game);
		}
	else if (!player->valid && game->timeout_observe)
		{
			timeout_del_pending_observe_request(game);
			sq_notify_wrong_version(game);
		}
	else if (player->valid && game->timeout_observe)
		{
			timeout_del_pending_observe_request(game);
			sqirc_send(connection,
								 "PRIVMSG %s :\001OPPONENTS\001\r\n", data->from.nick);
		}
}

/* :\001OPPONENTS\001
	 request the list of opponents
	 received from a probable observer */
SQIRC_FUNC_CMDCLIENT(cmd_ctcp_opponents)
{
	Player *player = find_player_by_nick(connection, preferences.nickname);
	GSList *list = player->opponents;
	char *opponent_nick;
	
	sqirc_send(connection,
						 "NOTICE %s :\001OPPONENTS_START\001\r\n", data->from.nick);
	
	while(list)
		{
			opponent_nick = (char *)list->data;
			sqirc_send(connection,
								 "NOTICE %s :\001OPPONENTS_ENTRY %s\001\r\n",
								 data->from.nick, opponent_nick);
			list = list->next;
		}
	
	sqirc_send(connection,
						 "NOTICE %s :\001OPPONENTS_END\001\r\n", data->from.nick);
}

/* :\001OPPONENTS_START
	 indicates the start of the opponents list for a player */
SQIRC_FUNC_CMDCLIENT(cmd_ctcp_opponentsstart)
{
	Player *player = find_player_by_nick(connection, data->from.nick);
	
	player->opponents_list = 1;
	
	if (player->opponents)
		g_slist_free(player->opponents);
	
	player->opponents = 0;
}

/* :\001OPPONENTS_END\001
	 indicates the end of the list of opponents */
SQIRC_FUNC_CMDCLIENT(cmd_ctcp_opponentsend)
{
	Player *player = find_player_by_nick(connection, data->from.nick);
	Game *game = 0;
	
	player->opponents_list = 0;
	game = find_game_by_opponent(connection, data->from.nick);
	
	if (game->pending_observe
			&& g_slist_length(player->opponents) > 0)
		game_observe_request(game);
	else
		{
			chatroom_chatmessage(CHATROOM(connection),
															 STARTOFF " \0033%s\003 has no games going on\n",
															 data->from.nick);
			game_cleanup(game);
		}
}

/* :\001OPPONENTS_ENTRY\001 <nick>
	 add an entry to the list opponents for an observer, sent once per name
	 the list of entries should be surrounded by 
	 OPPONENTS_START and OPPONENTS_END */
SQIRC_FUNC_CMDCLIENT(cmd_ctcp_opponentsentry)
{
	char *opponent_name;
	Player *player = find_player_by_nick(connection, data->from.nick);
	
	opponent_name = strdup(data->list[1]);
	opponent_name[strlen(opponent_name)-1] = '\0';   /* strip \001 from ctcp */
	
	if ( strcmp(preferences.nickname, opponent_name) != 0
			 && !find_player_opponent_by_nick(player, opponent_name))
		ADD_LIST_ITEM(player->opponents, opponent_name);
	else
		free(opponent_name);
	
}

/* :\001ACTION\001 <action>
	 issued by using /me <action>
   perform some action */
SQIRC_FUNC_CMDCLIENT(cmd_ctcp_action)
{
	Chatroom *chatroom;
	char *action = data->message+1;
	int c;
	
	while (*action != 0 && *action != ' ')
		action++;
	action++;            /* skip whitspace */
	
	/* delete the trailing \001 */
	for (c=0; *(action+c) != 0 && *(action+c) != 1; c++) {}
	*(action+c) = 0;
	
	chatroom = find_chatroom_by_channel(connection, data->nick);
	chatroom_chatmessage(chatroom, ACTIONMESSAGE, data->from.nick, action);
}

/* \001PLUGINS\001
	 another player wants to know what plugins we've loaded
	 in order to display appropriate game variation options */
SQIRC_FUNC_CMDCLIENT(cmd_ctcp_plugins)
{
	Game *game = 0;
	Player *player = 0;
	int answer = !strcmp("NOTICE", command);
	char *plugin_name = 0;
	time_t now = time(0);
	
	if (!answer)             /* privmsg: send out the plugins list */
		{
			GSList *list = sq_plugin_list();
			player = find_player_by_nick(connection, preferences.nickname);
			
			while (list)
				{
					Plugin *plugin = (Plugin *)list->data;
					sqirc_send(connection, "NOTICE %s :\001PLUGINS %s -\001\r\n",
										 data->from.nick, plugin->callbacks->name);
					list = list->next;
				}
			sqirc_send(connection, "NOTICE %s :\001PLUGINS END -\001\r\n",
								 data->from.nick);
			player->last_update = now;
			return;
		}
	
	/* handle the response */
	player = 0;
	if (!(player = find_player_by_nick(connection, data->from.nick)))
		{
			/* this should not happen!!! */
			player = sq_player_create(CHATROOM(connection));
			strcpy(player->nickname, data->from.nick);
			chatroom_debugmessage(" received a ctcp-plugins answer from an "
														"unknown player. this should not happen!\n");
		}
	player->last_update = now;
	player->flags |= PLAYER_HAVE_PLUGINS;
	
	if (strncmp(data->list[1], "END", 3) != 0)
		{
			plugin_name = strdup(data->list[1]);
			ADD_LIST_ITEM(player->plugins, plugin_name);
			
			/* not done with the list yet and wait for the next entry */
			return;
		}
	
	if ( (game = find_game_by_opponent(connection, data->from.nick)) )
		if (player->valid && game->timeout_play
				&& (player->flags & PLAYER_HAVE_VERSION) )
			{
				timeout_del_pending_play_request(game);
				game_play_request(game);
			}
}

/* ########################################################## */

/* PRIVMSG
	 received with a message containing the chat message */
SQIRC_FUNC_CMDCLIENT(cmd_client_privmsg)
{
	char *to = data->nick;
	char *message = data->message+1;
	Game *game = 0;
	Chatroom *chatroom = 0;
	
	/*	fprintf(stderr, "PRIVMSG to %s\n", to); */
	if (to[0] == '#' || to[0] == '&')
		{
			chatroom = find_chatroom_by_channel(connection, to);
			chatroom_chatmessage(chatroom,CHATMESSAGE, data->from.nick, message);
			return;
		}
	
	if ( (game = find_game_by_opponent(connection, data->from.nick)) )
		{
			game_chatmessage(game, CHATMESSAGE, data->from.nick, message);
			if (current_tablabel != game->tablabel)
				gtk_widget_set_style(game->tablabel, greenstyle);
		}
	else if ( (game = find_game_by_observer(connection, data->from.nick)) )
		{
			game_chatmessage(game, CHATMESSAGE, data->from.nick, message);
			if (current_tablabel != game->tablabel)
				gtk_widget_set_style(game->tablabel, greenstyle);
		}
	else if ( (game = find_game_by_observed(connection, data->from.nick)) )
		{
			game_chatmessage(game, CHATMESSAGE, data->from.nick, message);
			if (current_tablabel != game->tablabel)
				gtk_widget_set_style(game->tablabel, greenstyle);
		}
	else
		sq_print_on_current_chatwindow(PRIVMESSAGE, data->from.nick, message);
}

/* QUIT
	 a player has quit
	 cleanup anything associated with this player */
SQIRC_FUNC_CMDCLIENT(cmd_client_quit)
{
	/*
		Game *game = 0;
		
		game = find_game_by_opponent(connection, data->from.nick);
		if (game) 
		{
		game->game_over = TRUE;
		game_notify_quit(game);
		}
	*/
	sq_game_player_left(connection, data->from.nick, data->message+1);
	
}

/* PART
	 a player left the room/game */
SQIRC_FUNC_CMDCLIENT(cmd_client_part)
{
	char *channel = data->message;
	
	if (!data->from.nick[0])
		return;
	
	if (*channel == ':')
		channel++;
	if (!strcmp (data->from.nick, preferences.nickname))
		return;
	
	chatroom_chatmessage(CHATROOM(connection),
											 STARTON " Player \0033%s\003 has left the room\n",
											 data->from.nick);
	sq_game_player_left(connection, data->from.nick, NULL);
}

/* ERROR
	 server makes us aware of failures */
SQIRC_FUNC_CMDCLIENT(cmd_client_error)
{
	chatroom_chatmessage(CHATROOM(connection),
													 STARTON " %s", data->message+1);
	sqirc_close_connection(connection);
	return;
}

/* PING
	 received upon inactivity timeouts
	 immediately answered by a PONG */
SQIRC_FUNC_CMDCLIENT(cmd_client_ping)
{
	sqirc_send(connection, "PONG :%s\r\n", data->message+1);
}

/* JOIN
	 a player has joined */
SQIRC_FUNC_CMDCLIENT(cmd_client_join)
{
	Chatroom *chatroom;
	
	chatroom = find_chatroom_by_channel(connection, data->message+1);
	chatroom_chatmessage(chatroom,
											 STARTON " Player \0033%s\003 has joined %s\n",
											 data->from.nick, data->message+1);
	
	if ( strcmp(data->from.nick, preferences.nickname) != 0 )
		chatroom_player_add(chatroom, data->from.nick, 100);
	
	if (!strcasecmp(data->from.nick, preferences.nickname))
		{
			/* we are joining */
			sqirc_send(connection, "MODE %s\r\n", data->message+1);
			sqirc_send(connection, "WHO %s\r\n", data->message+1);
		}
	else
		{
			/* someone else is joining */
			sqirc_send(connection, "WHOIS %s\r\n", data->from.nick);
		}
}

/* NOTICE
	 this irc command is internally used by squaroid to exchange
	 client commands and to distinguish between commands and chat
	 normally only PRIVMSG's should be received for chatting */
SQIRC_FUNC_CMDCLIENT(cmd_client_notice)
{
	int i;                         /* counting is cool ;) */
	char name[BUFSIZE];
	struct client_command s_cmd;   /* squaroid command */
	char *message = data->message+1;
	
	if (!strncmp(data->nick, "AUTH", 4))
		{
			chatroom_chatmessage(CHATROOM(connection),
															 STARTOFF " %s\n", data->message+1);
			return;
		}

	if (message[0] == 1 && message[strlen (message) - 1] == 1)
		{
			struct client_command c_cmd; /* ctcp command */
			char cmd[BUFSIZE];
			int i;
			
			bzero(&c_cmd, sizeof(c_cmd));
			bzero(cmd, BUFSIZE);

			strncpy(cmd, message+1, strlen(message)-2);
			for (i=0;cmd[i] && cmd[i] != ' '; i++) {}
			cmd[i] = 0;
			
			for (i=0; ctcp_commands[i].command[0]; i++)
				{
					if (!strcmp(cmd, ctcp_commands[i].command))
						{
							c_cmd = ctcp_commands[i];
							break;
						}
				}
			
			if (!c_cmd.command[0])
				fprintf(stderr, "Unhandled CTCP command: '%s'\n", cmd);
			else
				c_cmd.callback(connection, "NOTICE", data);
			
			return;
		}
	
	bzero(&s_cmd, sizeof(s_cmd));
	strcpy(name,data->list[0]+1);
	for (i=0;name[i] && name[i] != ' '; i++) {}
	name[i] = 0;
	
	for (i=0; squaroid_commands[i].command[0]; i++)
		{
			if (!strcmp(squaroid_commands[i].command, name))
				{
					s_cmd = squaroid_commands[i];
					break;
				}
		}
	
	if (!s_cmd.command[0])
		sq_plugin_find_net_command(name, &s_cmd);
	
	if (!s_cmd.command[0])
		{
			chatroom_chatmessage(CHATROOM(connection),
													 STARTOFF " %s\n", data->message+1);
			fprintf(stderr, "Unhandled Squaroid Command: %s\n", name);
		}
	else
		s_cmd.callback(connection, name, data);


	return;
}

/* ############################################################# */

/* :UPDATEPLAYER <name> <score>
	 received by the bot to make us change to playerlist entry
	 of <player> */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_updateplayer)
{
	Player *player;
	
	player = find_player_by_nick(connection, data->list[1]);

	/* this can happen if a player leaves again, before the bot could even
		 out the UPDATEPLAYER command; and because i know this, i'll ignore it
		 kindly ;)
	*/
	if (!player)
		return;
	player->score = atoi(data->list[2]);
	
	chatroom_player_update(CHATROOM(connection), player);
}

/* :INVITE <first> <timeout> <timemax> <lag>
	 up to version 0.50-2 only <first> was sent
	 invite a player or adjust game settings
	 first contains YOU_FIRST, OPPONENT_FIRST, RANDOM_FIRST
	 timeout contains LIMIT_OFF, LIMIT_CUSTOM
	 timemax is 0 for timeout=LIMIT_OFF and >0 for LIMIT_CUSTOM
	 lag is the amount of time a CTCP-Version took to be answered */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_invite)
{
	Game *game = 0;
	
	if (preferences.auto_decline)
		{
			sqirc_send(connection, "NOTICE %s :INVITE_REJ\r\n", data->from.nick);
			chatroom_chatmessage(CHATROOM(connection),
													 STARTOFF " An invitation from %s has been "
													 "auto declined\n", data->from.nick);
			return;
		}
	
	if (!strcmp(command, "INVITE"))
		chatroom_chatmessage(CHATROOM(connection),
												 STARTOFF " you've been invited to play by "
												 "\0033%s\003\n", data->from.nick);
	else if (!strcmp(command, "INVITE_BUSY"))
		chatroom_chatmessage(CHATROOM(connection),
												 STARTOFF " %s seems to be currently inviting you!\n"
												 "    Try again later, or close your invitation window"
												 "\n    so that %s can invite you!\n",
												 data->from.nick, data->from.nick);
	
	if ( !(game = find_game_by_opponent(connection, data->from.nick)) )
		{
			game = game_create(connection);
			game->e_opponent = find_player_by_nick(connection, data->from.nick);
			strncpy(game->opponent, data->from.nick, sizeof(game->opponent));
		}
	
	/* error checking to keep backward compatibility as much as possible */
	if (!strcmp(command, "INVITE"))
		{
			game->options.first_player = atoi(data->list[1]);

			if (data->list[2] && data->list[3] && data->list[4] && data->list[5])
				{
					game->options.timeout = atoi(data->list[2]);
					game->movelimit.time_max = atoi(data->list[3]);
					game->e_opponent->lag = atoi(data->list[4]);
					GAME_PLUGIN(game) = sq_plugin_find_by_name(data->list[5]);
					chatroom_debugmessage(" INVITE [player=%d] [timeout=%d] "
																"[time_max=%d]\n",
																game->options.first_player,
																game->options.timeout,
																game->movelimit.time_max);
				}
			else
				{
					game->options.timeout = 0;
					game->movelimit.time_max = 0;
				}

			if (game->invited == NOT_INVITED)
				game->invited = BEEN_INVITED;

		}
	else
		game->invited = NOT_INVITED;
	
	if (sq_player_clientinfo(game->e_opponent) == TRUE
			&& (!data->list[5] || data->list[5][0] == '-'))
		game_play_request(game);
}

/* :INVITE_END
	 end of invitation sequence, now the actual game may start
	 this is necessary for a plugin-controlled game to get all configurations
	 over the net, before the actual invitation dialog pops up
	 this command is NOT sent, if there is no plugin-name sent along the
	 initial INVITE (i.e. plugin name was '-')
*/
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviteend)
{
	Game *game = 0;
	
	game = find_game_by_opponent(connection, data->from.nick);
	
	if ( game && sq_player_clientinfo(game->e_opponent) == TRUE )
		game_play_request(game);
}

/* :INVITE_ACK
	 the other side wants to play and accepts the game settings */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviteack)
{
	Game *game = 0;
	
	chatroom_chatmessage(CHATROOM(connection),
													 STARTOFF " your invitation to \0033%s\003 has "
													 "been accepted.\n", data->from.nick);
	
	game = find_game_by_opponent(connection, data->from.nick);

	if (!game)
		return;
	
	game->options.first_player = atoi(data->list[1]);	
	game_start_playing(game, GAME_NETWORK);
}

/* :INVITE_REJ
	 the other side does not want to play against us */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviterej)
{
	Game *game = 0;
	
	chatroom_chatmessage(CHATROOM(connection),
													 STARTOFF " your invitation to \0033%s\003 has "
													 "been rejected.\n", data->from.nick);
	
	if ( (game = find_game_by_opponent(connection, data->from.nick)) )
		game_cleanup(game);
	
	game = 0;
}

/* :GAMEQUIT
	 sent if somebody hits the 'Leave' button */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_gamequit)
{
	Game *game;
	time_t start = time(0);
	
	sq_game_all_del_observer(connection, data->from.nick);
  while ( (game = find_game_by_opponent(connection, data->from.nick))
					|| (game = find_game_by_observed(connection, data->from.nick)) )
		{
			game->game_over = TRUE;
			game_notify_quit(game);
			
			while (game->dialog_shown && start > time(0)-10)
				gtk_main_iteration();
		}
}

/* :GAMETIMEOUT
	 the other side timed out;
	 we check it and try to adjust for lag and
	 send an acknowledging :GAMETIMEOUT back
	 or we deny by sending an adjusting :TIMEOUTADJUST */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_gametimeout)
{
	Game *game;
	
	game = find_game_by_opponent(connection, data->from.nick);
	if (!game)
		return;

#if 0	
	if (game->movelimit.time_left > game->e_opponent->lag)
		{
			/* adjust for lag and inform opponent about it */
			/*
				game->movelimit.time_left += game->e_opponent->lag;
				sqirc_send(connection,
				"NOTICE %s :TIMEOUTADJUST %d %d\r\n",
				data->from.nick,
				game->e_opponent->lag,
				game->movelimit.time_left);
				game->movelimit.timed_out = FALSE;
				game->game_over = 0;
			*/
		}
	else
#endif
		if (!game->movelimit.timed_out)
			{
				game_chatmessage(game, STARTOFF " Timelimit exceeded!\n");
				sqirc_send(connection, "NOTICE %s :GAMETIMEOUT\r\n", data->from.nick);
				timeout_del_movelimit(game);
				game->movelimit.timed_out = TRUE;
				game->game_over = 1;
				game->playing = 0;
				game_is_over(game);
			}
	
}

/* :TIMEOUTADJUST <lag> <left>
	 adjust the time the to make a move for lag
   this is received only if we sent out :GAMETIMEOUT
   and the other side tells us we measured wrong */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_timeoutadjust)
{
	Game *game = find_game_by_opponent(connection, data->from.nick);

	if (!game)
		return;

	game->movelimit.timed_out = FALSE;
	game->movelimit.time_left = atoi(data->list[2]);
	
	game->e_opponent->lag += atoi(data->list[1]);
	game->game_over = 0;
	
	if (!game->movelimit.timed_out)
		timeout_add_movelimit(game);
}

/* :MOVE <x> <y>
	 or
	 :MOVE <XY>  (X being A to H)
	 set a stone at x,y position */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_move)
{
	Game *game;
	
	game = find_game_by_opponent(connection, data->from.nick);
	
	if (game == 0)
		game = find_game_by_observed(connection, data->from.nick);
	
	if (game != 0)
		{
			int x, y;
			
			/* MOVE A1 */
			x = data->list[1][0] - 'A';
			y = 7 - ((data->list[1][1] - '0') - 1);
			
			if (game->options.timeout == LIMIT_PERGAME
					&& data->list[2] && data->list[3])
				{
					game->movelimit.time_left_o = strtol(data->list[2], 0, 10);
					game->movelimit.time_left = strtol(data->list[3], 0, 10);
				}
			
			game_canvas_put_stone(game, x, y);
			if (current_tablabel != game->tablabel)
				gtk_widget_set_style(game->tablabel, greenstyle);
		}
	else
		sqirc_send(connection, "NOTICE %s :GAMEQUIT\r\n", data->from.nick);
	
}

/* :INVITE_AGAIN_ACK
	 the opponent wants to play again */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviteagainack)
{
	Game * game = find_game_by_opponent(connection, data->from.nick);
	if (game)
		game->game_over = FALSE;
}

/* :INVITE_AGAIN_REJ
	 the opponent denies to play again */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_inviteagainrej)
{
	Game *game = find_game_by_opponent(connection, data->from.nick);
	
	if (game)
		game_notify_quit(game);
}

/* :OBSERVE <nick>
	 receiving this means that somebody wants to watch us play
	 against <nick> */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observe)
{
	Game *game = find_game_by_opponent(connection, data->list[1]);

	if (!preferences.observe_decline
			&& game
			&& g_slist_length(game->observer) < 5
			&& !find_game_observer_by_nick(game, data->from.nick))
		{
			char *move_data = game_encode_board_data(game);
			
			sqirc_send(connection,
								 "NOTICE %s :OBSERVE_ACK %s %d %s\r\n",
								 data->from.nick, data->list[1],
								 game->current_move, move_data);
			free(move_data);
			
			sqirc_send(connection,
								 "NOTICE %s :OBSERVE_ADD %s\r\n",
								 game->opponent, data->from.nick);
			
			game_chatmessage(game,
													 STARTON " Observer \003%s\003 entered the "
													 "game\n", data->from.nick);
			sq_game_add_observer(game, data->from.nick);
		}
	else
		sqirc_send(connection, "NOTICE %s :OBSERVE_REJ\r\n", data->from.nick);
}

/* :OBSERVE_ACK
	 accept a request for observation */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observeack)
{
	Game *game = find_game_by_opponent(connection, data->from.nick);
	
	assert(game != 0);
	strcpy(game->observed, data->list[1]);
	game->current_move = atoi(data->list[2]);
	
	game_decode_board_data(game, data->list[3]);
	game_start_playing(game, GAME_OBSERVE);
}

/* :OBSERVE_REJ
	 reject a request for observation */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observerej)
{
	Game *game = 0;

	chatroom_chatmessage(CHATROOM(connection),
											 STARTOFF " your observe request to \0033%s\003 "
											 "has been rejected.\n", data->from.nick);

	if ( (game = find_game_by_opponent(connection, data->from.nick)) )
		game_cleanup(game);
	game = 0;
}

/* :OBSERVE_ADD <nick>
	 add <nick> to the list of observers
	 this is received by our immediate opponent, telling us that
	 he received an observation request */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observeadd)
{
	Game *game = 0;
	
	game = find_game_by_opponent(connection, data->from.nick);
	assert(game != 0);
	
	game_chatmessage(game, STARTON " Observer \0033%s\003 entered the game\n",
									 data->list[1]);
	
	sq_game_add_observer(game, data->list[1]);
}

/* :OBSERVE_RESET
	 request received by an observed telling him to clear the board
	 and reset game values */
SQIRC_FUNC_CMDCLIENT(cmd_squaroid_observereset)
{
	Game *game = find_game_by_opponent(connection, data->from.nick);
	
	if (!game
			&& !(game = find_game_by_observed(connection, data->from.nick)))
		return;
	
	game_reset_values(game);
	board_draw_background(game);
}

SQIRC_FUNC_CMDCLIENT(cmd_squaroid_playerid)
{
	snprintf(preferences.playerid, 33, "%s", data->list[1]);
}
