#include "squaroid.h"
#include <dlfcn.h>

static GSList *plugins = 0;
extern SquaroidPlugin *sq_builtin_plugins[];

GSList *sq_plugin_list()
{
	return plugins;
}

Plugin *sq_plugin_create()
{
	Plugin *plugin = malloc(sizeof(Plugin));
	
	bzero(plugin, sizeof(Plugin));
	return plugin;
}

void sq_plugin_cleanup()
{
	while (plugins)
		{
			Plugin *plugin = (Plugin *)plugins->data;
			
			DEL_LIST_ITEM(plugins, plugin);
			
			if (plugin->callbacks->OnUnload != 0)
				plugin->callbacks->OnUnload();

			if (plugin->handle)
				dlclose(plugin->handle);
			free(plugin);
		}
}

int sq_plugin_init()
{
	int i;
	for (i = 0; sq_builtin_plugins[i]; i++)
		{
			Plugin *plugin = sq_plugin_create();
			plugin->handle = 0;
			plugin->get_module = 0;
			plugin->callbacks = sq_builtin_plugins[i];

			if ( (sq_plugin_find_by_name(plugin->callbacks->name)) != 0)
				{
					printf("Plugin '%s' already loaded\n", plugin->callbacks->name);
					if (plugin->handle)
						dlclose(plugin->handle);
					free(plugin);
					continue;
				}
			
			if (plugin->callbacks->OnLoad != 0)
				plugin->callbacks->OnLoad(plugin);
			else
				printf("Plugin '%s' successfully loaded\n", plugin->callbacks->name);
			
			ADD_LIST_ITEM(plugins, plugin);
		}
	return 0;
}

int sq_plugin_load(const char *filename)
{
	if (!access(filename, R_OK))
		{
			Plugin *plugin = sq_plugin_create();
			
			plugin->handle = dlopen(filename, RTLD_LAZY);
			
			if (!plugin->handle)
				{
					fprintf(stderr, "Error opening library (%s): %s\n",
									filename, dlerror());
					free(plugin);
					return -1;
				}
			
			plugin->get_module = dlsym(plugin->handle, "get_module");
			plugin->callbacks = plugin->get_module();

			if ( (sq_plugin_find_by_name(plugin->callbacks->name)) != 0)
				{
					printf("Plugin '%s' already loaded\n", plugin->callbacks->name);
					if (plugin->handle)
						dlclose(plugin->handle);
					free(plugin);
					return 0;
				}
			
			if (plugin->callbacks->OnLoad != 0)
				plugin->callbacks->OnLoad(plugin);
			else
				printf("Plugin '%s' successfully loaded\n", plugin->callbacks->name);
			
			ADD_LIST_ITEM(plugins, plugin);
			
			return 0;
		}
	else
		fprintf(stderr, "Permission denied. Could not load plugin %s\n", filename);
	return -1;
}

Plugin *sq_plugin_find_by_name(const char *name)
{
	GSList *list = plugins;
	
	while (list)
		{
			if (!strcasecmp(((Plugin*)list->data)->callbacks->name, name))
				return (Plugin *)list->data;
			list = list->next;
		}
	return NULL;
}

void sq_plugin_find_net_command(const char *name, struct client_command *cmd)
{
	GSList *list = plugins;
	
	while (list)
		{
			Plugin *plugin = (Plugin*)list->data;
			int i;

			if (plugin->callbacks->commands == 0)
				{
					list = list->next;
					continue;
				}
			
			for (i=0; plugin->callbacks->commands[i].command[0]; i++)
				{
					if (!strcmp(plugin->callbacks->commands[i].command, name))
						{
							memcpy(cmd, &plugin->callbacks->commands[i], sizeof(*cmd));
							return;
						}
				}
			
			list = list->next;
		}
}
