#ifndef SQ_HAVE_PLUGIN_H
#define SQ_HAVE_PLUGIN_H

#define PLUGIN_DONT_ANNOUNCE 1<<0

#define PLUGIN_FAILURE       0
#define PLUGIN_SUCCESS       1

typedef struct plugin Plugin;
typedef struct squaroid_plugin SquaroidPlugin;

struct squaroid_plugin
{
	const char *name;
	ulong flags;                              
	
	struct client_command *commands;          /* plugin net commands */
	ulong (*OnLoad)(Plugin *);                /* plugin just loaded */
	ulong (*OnUnload)();                      /* plugin is to be unloaded */
	
	ulong (*OnInvite)(Game *, GtkWidget **);  /* have invited or been invited */
	ulong (*OnInviteSend)(Game *);            /* send plugin data over irc */
	ulong (*OnGameStart)(Game *);             /* the game just started */
	ulong (*OnMove)(Game *, int, int);        /* a player made a move */
	ulong (*OnSquare)(Game *, int);           /* a square was made */
	ulong (*OnGameOver)(Game *);              /* the game is over */
	ulong (*OnCleanup)(Game *);               /* cleanup whatever necessary */
	
};

struct plugin
{
	void *handle;
	SquaroidPlugin *(*get_module)();
	SquaroidPlugin *callbacks;
	void *data;
};


int sq_plugin_init();
GSList *sq_plugin_list();
int sq_plugin_load(const char *);
void sq_plugin_cleanup();
Plugin *sq_plugin_find_by_name(const char *);
void sq_plugin_find_net_command(const char *, struct client_command *);

#endif /* SQ_HAVE_PLUGIN_H */
