/* Squaroid
 * Copyright (C) 1999,2000 Lukas Schrder
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */


#include "squaroid.h"
#include <getopt.h>
#include <signal.h>

static int fd_sndserver = 0;        /* FD to communicate with sound child */

static gboolean disable_game;

static GtkWidget *w_connect_dialog = 0;     /* connect dialog widget */
static GtkWidget *iname = 0,                /* entry widget: nickname */      
	*ihost = 0,                               /* entry widget: hostname */
	*iport = 0,                               /* entry widget: port */
	*ignore_motd = 0;                         /* checkbox: ignore motd */
static GdkColor black;
static GdkColor white;
static GdkFont *font = 0;

extern struct server_command server_commands[];
extern struct client_command client_commands[];
extern struct client_command ctcp_commands[];

static void callback_connect(GtkWidget *, gpointer);

#ifdef void
#error "VOID DEFINED - what the heck?"
#endif

void sq_get_gfx_path(char *file, char *target)
{
	sprintf(target, "%s/%s/%s", preferences.gfxdir, preferences.gfxtheme, file);
}

GtkStyle *sq_get_widget_style()
{
	GtkStyle *style = gtk_style_new();
	
	style->base[GTK_STATE_NORMAL] = colors[1];
	style->bg[GTK_STATE_NORMAL] = colors[1];
	
	return style;
}

int sq_check_files()
{
	char *file = malloc(512);
	char *buffer = malloc(1024);
	int fd;
	int failed = 0;
	
#define GFX_CHECK(gCHK)                                                   \
	 GFXPATH(gCHK,file);                                                    \
     if ( (fd = open(file,O_RDONLY) == -1))                               \
       {                                                                  \
         fprintf(stderr, "%s: %s\n",file,strerror(errno));                \
		     failed++;                                                        \
       }
	
	GFX_CHECK("theme.conf");
	
	if (!failed && !load_theme_config())
		{
			GFX_CHECK(preferences.theme_data.invite);
			GFX_CHECK(preferences.theme_data.observe);
			GFX_CHECK(preferences.theme_data.background);
			GFX_CHECK(preferences.theme_data.p1_small);
			GFX_CHECK(preferences.theme_data.p1_last);
			GFX_CHECK(preferences.theme_data.p1_stone);
			GFX_CHECK(preferences.theme_data.p2_small);
			GFX_CHECK(preferences.theme_data.p2_last);
			GFX_CHECK(preferences.theme_data.p2_stone);
			GFX_CHECK(preferences.theme_data.marker);
		}
	else
		fprintf(stderr,
						"******************************************\n"
						"Could not load theme configuration\n"
						"******************************************\n");
	//		chatroom_chatmessage(NULL, "Could not load theme configuration\n");
	
	free(file);
	free(buffer);
	return failed;
}

void callback_connectdialogdestroy(GtkWidget *w, gpointer data)
{
	gtk_widget_destroy(w_connect_dialog);
	iport = 0;
	ihost = 0;
	iname = 0;
	ignore_motd = 0;
	w_connect_dialog = 0;
}

void callback_start_sologame(GtkWidget *w, gpointer data)
{
	 callback_connectdialogdestroy(w_connect_dialog, 0);
	 menucallback_start_sologame(w, data);
}

void callback_start_aigame(GtkWidget *w, gpointer data)
{
	callback_connectdialogdestroy(w_connect_dialog, 0);
	menucallback_start_aigame(w, data);
}

void callback_dialogdestroy(GtkWidget *w, gpointer data)
{
	gtk_widget_destroy(w);
}

void show_dialog_connect()
{
	GtkWidget *table =0, *label = 0, *connect = 0, *quit = 0, *solo = 0;
	GtkTooltips *tooltip;
	char buf[256];

	bzero(buf, 256);
	if (sq_check_files())
		{
			fprintf(stderr, "\nThere are files missing!\n"
							"Check your preferences for correctness\n");
			disable_game = TRUE;
			callback_destroy(NULL, NULL);
			return;
		}
	else
		disable_game = FALSE;
	
	tooltip = gtk_tooltips_new();

	w_connect_dialog = gtk_dialog_new();	
	gtk_window_set_position(GTK_WINDOW(w_connect_dialog), GTK_WIN_POS_CENTER);
	gtk_widget_set_usize(w_connect_dialog, 400, 200);
	gtk_window_set_title(GTK_WINDOW(w_connect_dialog),
											 "Connect to Squaroid-Server");
	gtk_signal_connect(GTK_OBJECT(w_connect_dialog),
										 "delete_event",
										 GTK_SIGNAL_FUNC(callback_connectdialogdestroy),
										 NULL);

	table = gtk_table_new(4,2,FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
	gtk_container_border_width(GTK_CONTAINER(table), 5);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w_connect_dialog)->vbox), table,
										 TRUE, TRUE, 0);

	label = gtk_label_new("Name:");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 0, 0, 0, 0);

	iname = gtk_entry_new_with_max_length(9);
	gtk_entry_set_text(GTK_ENTRY(iname), preferences.nickname);
	gtk_table_attach_defaults(GTK_TABLE(table), iname, 1, 3, 0, 1);
	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), iname,
											 "Enter your preferred nickname here",
											 "Enter your preferred nickname here");

	label = gtk_label_new("Host:");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 0, 0);

	ihost = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(ihost), preferences.hostname);
	gtk_table_attach_defaults(GTK_TABLE(table), ihost, 1, 3, 1, 2);
	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), ihost,
											 "Enter the hostname of the IRC server to play on",
											 "Enter the hostname of the IRC server to play on");

	label = gtk_label_new("Port:");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 0, 0, 0, 0);

	sprintf(buf, "%d", preferences.port);

	iport = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(iport), buf);
	gtk_table_attach_defaults(GTK_TABLE(table), iport, 1, 3, 2, 3);
	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), iport,
											 "Enter the port to connect to (Default: 6667)",
											 "Enter the port to connect to");

	ignore_motd = gtk_check_button_new_with_label("Skip MOTD");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ignore_motd),
															 preferences.ignore_motd);
	gtk_table_attach(GTK_TABLE(table), ignore_motd, 2, 3, 3, 4, 0, 0, 0, 0);
	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltip), ignore_motd,
											 "Skip IRC server's Message Of The Day",
											 "Skip IRC server's Message Of The Day");

	connect = gtk_button_new_with_label("Connect");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w_connect_dialog)->action_area),
										 connect, TRUE, TRUE, 0);
	solo = gtk_button_new_with_label("Solo Game");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w_connect_dialog)->action_area), solo,
										 TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(solo), "clicked",
										 GTK_SIGNAL_FUNC(callback_start_sologame), NULL);
	solo = gtk_button_new_with_label("CPU Game");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w_connect_dialog)->action_area), solo,
										 TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(solo), "clicked",
										 GTK_SIGNAL_FUNC(callback_start_aigame), NULL);
	quit = gtk_button_new_with_label("Quit");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w_connect_dialog)->action_area), quit,
										 TRUE, TRUE, 0);

	gtk_signal_connect(GTK_OBJECT(quit), "clicked",
										 GTK_SIGNAL_FUNC(callback_destroy), NULL);
	gtk_signal_connect(GTK_OBJECT(connect), "clicked",
										 GTK_SIGNAL_FUNC(callback_connect), NULL);
	gtk_signal_connect(GTK_OBJECT(iname), "activate",
										 GTK_SIGNAL_FUNC(callback_connect), NULL);
	gtk_signal_connect(GTK_OBJECT(ihost), "activate",
										 GTK_SIGNAL_FUNC(callback_connect), NULL);
	gtk_signal_connect(GTK_OBJECT(iport), "activate",
										 GTK_SIGNAL_FUNC(callback_connect), NULL);

	gtk_widget_grab_focus(iname);

	gtk_widget_show_all(GTK_DIALOG(w_connect_dialog)->vbox);
	gtk_widget_show_all(GTK_DIALOG(w_connect_dialog)->action_area);

	gtk_widget_show(w_connect_dialog);
}

void callback_connect(GtkWidget *w, gpointer data)
{
	char *version = VERSION;
	Player *player = 0;
	struct connection *connection;
	Chatroom *chatroom;

	strcpy(preferences.nickname, gtk_entry_get_text(GTK_ENTRY(iname)));
	strcpy(preferences.hostname, gtk_entry_get_text(GTK_ENTRY(ihost)));
	preferences.port = atoi(gtk_entry_get_text(GTK_ENTRY(iport)));
	preferences.ignore_motd
		= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ignore_motd));

	if ( (chatroom = find_unconnected_chatroom()) == 0)
		ADD_LIST_ITEM(chatroom_list, chatroom_create(NULL));

	if ( (connection = connect_to_server()) != 0)
		{
			/* create a new player for us now... */
			player = sq_player_create(CHATROOM(connection));

			strcpy(player->nickname,preferences.nickname);
			player->major_version = atoi(version);
			player->minor_version = atoi(version+2);
			player->last_update = time(0);

		}
	callback_connectdialogdestroy(0, 0);
}

void sq_connection_cleanup(struct connection *connection)
{

	/* say goodbye to everyone */
	sqirc_send(connection, "QUIT :bye bye\r\n");

	/*
		in case something went wrong during connection, there's nothing
		here to cleanup for us; so get out!
	*/
	if (CHATROOM(connection) == 0)
		return;
	
  chatroom_player_clearlist(CHATROOM(connection));
	
	/* cleanup all games runnong on this connection */
	while ( CHATROOM(connection)->games )
		{
			Game *game = (Game *)CHATROOM(connection)->games->data;
			DEL_LIST_ITEM(CHATROOM(connection)->games, game);
			game_cleanup(game);
		}
	
	/* cleanup all subrooms associated with this connection */
	while ( CHATROOM(connection)->rooms )
		chatroom_destroy((Chatroom*)CHATROOM(connection)->rooms->data);
	
	/* finally clear the list of players on this server connection */
	sq_player_cleanup_all(CHATROOM(connection));
	
	CHATROOM(connection)->players = 0;
	CHATROOM(connection)->rooms = 0;
	CHATROOM(connection)->games = 0;
	
	gtk_label_set_text(GTK_LABEL(CHATROOM(connection)->tablabel), "<none>");
	chatroom_chatmessage(CHATROOM(connection),
											 STARTON " Connection closed...\n");
	CHATROOM(connection)->connection = 0;
}

void callback_destroy(GtkWidget *widget, gpointer data)
{
  if (preferences.saveonexit)
    save_config();
	sqirc_close_all_connections();
	
	sq_finalize_soundserver();
	sq_plugin_cleanup();
  gtk_main_quit();
}

Player *sq_player_create(Chatroom *chatroom)
{
  Player *player = 0;
	
	errno = 0;
	player = malloc(sizeof(Player));
  bzero(player, sizeof(Player));
  
  player->major_version = 0;
  player->minor_version = 0;
  player->last_update = 0;
	player->score = 100;                /* update score */
	player->connection = chatroom->connection;
	
	/* get the chatroom this way, to always reference the main chatroom
		 of a connection, and not some subroom */
  ADD_LIST_ITEM(CHATROOM(player->connection)->players, player);
  return player;
}

/*
	interpret or request clientinfo, and return TRUE if it's ok to
	play this player, FALSE if it is not, or clientinfo had to be requested
*/
gboolean sq_player_clientinfo(Player *player)
{
	Game *game = 0;
	
	/* we have everything necessary */
	if ( (player->flags & PLAYER_HAVE_VERSION) == PLAYER_HAVE_VERSION
			 && (player->flags & PLAYER_HAVE_PLUGINS) == PLAYER_HAVE_PLUGINS)
		return player->valid;
	
	/*
	 * if code execution actually gets here, some needed information
	 * must be missing
	 */
	
	/* request version information if we dont have it yet */
	if ( (player->flags & PLAYER_HAVE_VERSION) == 0)
		net_send_ctcp_version(player->connection, player->nickname);
	
	/* request plugins information if we dont have it yet */
	if ( (player->flags & PLAYER_HAVE_PLUGINS) == 0)
		sqirc_send(player->connection,
							 "PRIVMSG %s :\001PLUGINS\001\r\n",player->nickname);
	
	/* this is necessary, to measure the lag to this player */
	player->last_update = time(0);
	
	game = find_game_by_opponent(player->connection, player->nickname);
	timeout_add_pending_play_request(game);	
	return FALSE;
}

void sq_player_cleanup(Player *player)
{
	DEL_LIST_ITEM(CHATROOM(player->connection)->players, player);
	sq_cleanup_simple_glist(player->plugins);
  free(player);
}

void sq_player_cleanup_all(Chatroom *chatroom)
{
  Player *player;
  char *opponent_name;
	
	if (chatroom == 0)
		chatroom = find_chatroom_by_tablabel(0);
	
  while (chatroom->players)
		{
			player = (Player *)(chatroom->players)->data;
			while (player->opponents)
				{
					opponent_name = (char *)player->opponents->data;
					DEL_LIST_ITEM(player->opponents, opponent_name);
					free(opponent_name);
				}
			sq_player_cleanup(player);
		}
}

void sq_cleanup_simple_glist(GSList *list)
{
	while (list)
		{
			void *data = list->data;
			DEL_LIST_ITEM(list, data);
			free(data);
		}
}

Player *find_player_by_nick(struct connection *connection, const char *nick)
{
  Player *player = 0;
  GSList *list = 0;
	
	/* connection == 0 when starting SOLO or CPU game */
	if (connection == 0)
		return 0;
	
	list = CHATROOM(connection)->players;
	
  while (list)
		{
			player = (Player *)list->data;
			if (!strcmp(nick, player->nickname))
				return player;
			list = list->next;
		}
  return 0;
}

char *find_player_opponent_by_nick(Player *player, char *nick)
{
  char *opponent;
  GSList *list = player->opponents;
	
  while (list)
    {
      opponent = (char *)list->data;
      if (!strcmp(nick, opponent))
				return opponent;
      list = list->next;
    }
  return 0;
}

gboolean sq_player_has_plugin(Player *player, const char *plugin)
{
	GSList *list = player->plugins;
	
	while (list)
		{
			if (!strcasecmp((char*)list->data, plugin))
				return TRUE;
			list = list->next;
		}
	return FALSE;
}

char *find_game_observer_by_nick(Game *game, char *nick)
{
  GSList *list = game->observer;
  char *observer;
  while (list)
    {
      observer = (char *)list->data;
      if (!strcmp(nick, observer))
				return observer;
      list = list->next;
    }
  return 0;
}

void sq_game_add_observer(Game *game, char *observer)
{
  char *new_observer = malloc(strlen(observer)+1);

  strcpy(new_observer, observer);
  ADD_LIST_ITEM(game->observer, new_observer);
}

void sq_game_del_observer(Game *game, char *observer)
{
  char *old_observer = 0;
	
  if ( (old_observer = find_game_observer_by_nick(game, observer)) )
    {
			DEL_LIST_ITEM(game->observer, old_observer);
			
			game_chatmessage(game, STARTON " Observer \0033%s\003 left\n",
											 observer);
			free(old_observer);
    }
}

void sq_game_all_del_observer(struct connection *connection, char *nick)
{
  GSList *g_list = CHATROOM(connection)->games;
	
  while (g_list)
    {
      sq_game_del_observer((Game *)g_list->data, nick);
      g_list = g_list->next;
    }
}

void sq_game_player_left(struct connection *connection,
												 char *nick, char *reason)
{
  Game *game;
  Player *player;
	GSList *list = 0;
	
	chatroom_player_remove(CHATROOM(connection), nick, reason);

  sq_game_all_del_observer(connection, nick);
  while ( (game = find_game_by_opponent(connection, nick))
					|| (game = find_game_by_observed(connection, nick)) )
		{
			game_notify_quit(game);
			while (game->dialog_shown)
				gtk_main_iteration();
		}
	
  if ( (player = find_player_by_nick(connection, nick)) )
		sq_player_cleanup(player);

	/* maybe the gamehost left... */
	if (!strcasecmp(CHATROOM(connection)->gamehost, nick))
		{
			CHATROOM(connection)->gamehost[0] = 0;
			
			/* try to find a fallback host */
			list = CHATROOM(connection)->players;
			while (list)
				{
					player = (Player *)list->data;
					if (strstr(player->realname, "Squaroid Host"))
						{
							strncpy(CHATROOM(connection)->gamehost, player->nickname,
											SMALLBUF);
							chatroom_chatmessage(CHATROOM(connection),
																	 STARTOFF " found HostBot \0032%s\003\n",
																	 player->nickname);
							break;
						}
					list = list->next;
				}
		}
}

void sq_game_observer_notify(Game *game, char *type, char *message)
{
  GSList *list = game->observer;
  char rcpt[BUFSIZE];
  int count = 0;
  memset(rcpt, 0, sizeof(rcpt));
  while (list)
    {
      if (!strlen(rcpt))
				strcpy(rcpt, (char *)list->data);
      else 
				strcat(rcpt, (char *)list->data);
      list = list->next;
      if (list && !(count % 5))
				{
					sqirc_send(game->connection,
										 "%s %s :%s\r\n", type, rcpt, message);
					memset(rcpt, 0, sizeof(rcpt));
				}
      else if (list) strcat(rcpt, ",");
      count++;
    }
  if (strlen(rcpt))
		sqirc_send(game->connection, "%s %s :%s\r\n", type, rcpt, message);
}

Game *find_game_by_opponent(struct connection *connection,
														const char *opponent)
{
  Game *game;
  GSList *list = CHATROOM(connection)->games;
	
  if (!list)
		return 0;
	
  while(list)
		{
			game = (Game *)list->data;
			assert(game != 0);
			if (!strcmp(opponent, game->opponent))
				return game;
			list = list->next;
		}
  return 0;
}

Game *find_game_by_observed(struct connection *connection,
														const char *observed)
{
  Game *game;
  GSList *list = CHATROOM(connection)->games;

  if (list == 0)
		return 0;
  
  while(list)
		{
			game = (Game *)list->data;
			if (!strcmp(observed, game->observed))
				return game;
			list = list->next;
		}
  return 0;
}

Game *find_game_by_canvas(GtkWidget *canvas)
{
  Game *game;
	GSList *connections = chatroom_list;
  GSList *list;

	while (connections)
		{
			list = ((Chatroom*)connections->data)->games;
			while(list)
				{
					game = (Game *)list->data;
					if (canvas == game->canvas)
						return game;
					list = list->next;
				}
			connections = connections->next;
		}
			return 0;
}

Game *find_game_by_vbox(GtkWidget *vbox)
{
  Game *game;
	GSList *connections = chatroom_list;
  GSList *list;
  assert(vbox != 0);
  
	while (connections)
		{
			list = ((Chatroom*)connections->data)->games;
			while(list)
				{
					game = (Game *)list->data;
					if (vbox == game->vbox)
						return game;
					list = list->next;
				}
			connections = connections->next;
		}
  
  return 0;
}

Game *find_game_by_tablabel(GtkWidget *tablabel)
{
	Game *game;
	GSList *connections = chatroom_list;
  GSList *list;
  
	while (connections)
		{
			list = ((Chatroom*)connections->data)->games;
			while(list)
				{
					game = (Game *)list->data;
					if (tablabel == game->tablabel)
						return game;
					list = list->next;
				}
			connections = connections->next;
		}
  
  return 0;
}

Game *find_game_by_dialog(GtkWidget *dialog)
{
	Game *game;
	GSList *connections = chatroom_list;
	GSList *list;
	
	while (connections)
		{
			list = ((Chatroom *)connections->data)->games;
			while(list)
				{
					game = (Game *)list->data;
					if (game->dialog == dialog)
						return game;
					list = list->next;
				}
			connections = connections->next;
		}
	return 0;
}

Game *find_game_by_observer(struct connection *connection,
														char *nick)
{
	Game *game;
  GSList *list = CHATROOM(connection)->games;
  GSList *o_list;

  while(list) 
    {
      game = (Game *)list->data;
      o_list = game->observer;
      while (o_list)
				{
					if (!strcmp((char *)o_list->data, nick))
						return game;
					o_list = o_list->next;
				}
      list = list->next;
    }
  return 0;
}

Chatroom *find_chatroom_by_table(GtkWidget *table)
{
	Chatroom *chatroom;
	Chatroom *room;
	GSList *list = chatroom_list;
	GSList *rooms = 0;
	
	while (list)
		{
			chatroom = (Chatroom *)list->data;

			if (table == chatroom->table)
				return chatroom;

			rooms = chatroom->rooms;
			while (rooms)
				{
					room = (Chatroom *)rooms->data;
					if (table == room->table)
						return room;
					rooms = rooms->next;
				}
			list = list->next;
		}
	return 0;
}

Chatroom *find_chatroom_by_tablabel(GtkWidget *tablabel)
{
	Chatroom *chatroom;
	GSList *list = chatroom_list;
	
	while (list)
		{
			chatroom = (Chatroom *)list->data;
			if (tablabel == chatroom->tablabel)
				return chatroom;
			list = list->next;
		}
	return 0;
}

Chatroom *find_chatroom_by_connection(struct connection *connection)
{
	Chatroom *chatroom;
	GSList *list = chatroom_list;
	while (list)
		{
			chatroom = (Chatroom *)list->data;
			if (connection == chatroom->connection)
				return chatroom;
			list = list->next;
		}
	return 0;
}

Chatroom *find_chatroom_by_channel(struct connection *connection,
																	 const char *channel)
{
	Chatroom *chatroom;
	GSList *list = CHATROOM(connection)->rooms;

	if (!strcasecmp(channel, CHATROOM(connection)->channel))
		return CHATROOM(connection);

	while (list)
		{
			chatroom = (Chatroom*)list->data;
			if (!strcasecmp(channel, chatroom->channel))
				return chatroom;
			list = list->next;
		}
	return 0;
}

Chatroom *find_unconnected_chatroom()
{
	Chatroom *chatroom;
	GSList *list = chatroom_list;
	while (list)
		{
			chatroom = (Chatroom *)list->data;
			if (chatroom->connection == 0 && chatroom->tablabel != 0)
				return chatroom;
			list = list->next;
		}
	return 0;
}

void sq_status_set(char *fmt, ...)
{
	char st_buffer[BUFSIZE];
	va_list ap;
	
	sq_status_unset();
	va_start(ap, fmt);
	vsnprintf(st_buffer, BUFSIZE, fmt, ap);
	va_end(ap);
	
  gtk_statusbar_push(GTK_STATUSBAR(top_statusbar), 1, st_buffer);
}

void sq_status_unset()
{
  gtk_statusbar_pop(GTK_STATUSBAR(top_statusbar), 1);
}

/* this function notifies the user, that the opponent
 * does not have a valid client
 */
void sq_notify_wrong_version(Game *game)
{
  chatroom_chatmessage(CHATROOM(game->connection),
													 STARTON " Player \0033%s\003 does not have a valid "
													 "Squaroid version!\n", game->opponent);
  game_cleanup(game);
}

void sq_notify_wrong_version_player(Player *player)
{
  sq_print_on_current_chatwindow(STARTON " Player \0033%s\003 does not have a "
																 "valid Squaroid version!\n",
																 player->nickname);
	sq_player_cleanup(player);
}

void sq_print_on_current_chatwindow(char *fmt, ...)
{
	va_list args;
	char buf[BUFSIZE];
	Game *game;
	Chatroom *chatroom;

	va_start(args, fmt);
	vsnprintf(buf, BUFSIZE, fmt, args);
	va_end(args);
	
	if ( (game = find_game_by_tablabel(current_tablabel)) )
		game_chatmessage(game, "%s", buf);
	else if ( (chatroom = find_chatroom_by_tablabel(current_tablabel)) )
		chatroom_chatmessage(chatroom, "%s", buf);
	
}

void sq_initialize_soundserver()
{
#if SOUND
	int snd;
	fprintf(stderr, "Initiliazing sound server: ");
	
	snd = sq_init_sound(DEFAULT_SFXDIR, DEFAULT_SFXTHEME, &fd_sndserver);
	fprintf(stderr, "%s\n", (snd==0)?"success":"failure");
#endif
}

void sq_play_sound(const char *sound)
{
#if SOUND
	char cmd[256];

	if (!preferences.sound)
		return;

	if (fd_sndserver <= 0)
		return;
	
	bzero(cmd, 256);
	snprintf(cmd, 256, "play %s\n", sound);
	sq_stop_sound();
	write(fd_sndserver, cmd, strlen(cmd));
#endif
}

/* play the sound most appropriate for score */
void sq_play_square_sound(int score)
{
#if SOUND
	if (!preferences.sound)
		return;

	if (score >= 64)
		sq_play_sound("sq_64");
	else if (score >= 49 && score < 64)
		sq_play_sound("sq_49");
	else if (score >= 32 && score < 49)
		sq_play_sound("sq_32");
	else if (score >= 25 && score < 32) 
		sq_play_sound("sq_25");
	else if (score >= 18 && score < 25)
		sq_play_sound("sq_18");
	else if (score >= 16 && score < 18)
		sq_play_sound("sq_16");
	else if (score >= 9 && score < 16)
		sq_play_sound("sq_9");
	else if (score > 4 && score <= 8)
		sq_play_sound("sq_8");
	else if (score == 4)
		sq_play_sound("sq_4");
#endif	
}

void sq_stop_sound()
{
#if SOUND
	if (!preferences.sound)
		return;

	if (fd_sndserver <= 0)
		return;
	
	write(fd_sndserver, "stop\n", 5);
#endif
}

void sq_finalize_soundserver()
{
#if SOUND
	if (!preferences.sound)
		return;

	if (fd_sndserver <= 0)
		return;

	write(fd_sndserver, "quit\n", 5);
	close(fd_sndserver);
	fd_sndserver = -1;
#endif
}

void sq_print_version()
{
	printf("Squaroid v%s\n", VERSION);
	printf("  compilation date: %s, %s\n\n", __TIME__, __DATE__);
	printf("Configuration:\n");
#if SOUND
  printf("  sound support   : enabled\n");
	printf("  sound device    : %s\n", DEFAULT_SFXDEV);
#else
  printf("  sound support   : disabled\n");
#endif
	printf("\n");
}

void sq_print_help()
{
#define HLP(_item, _desc) printf("  %-15s    %s\n", _item, _desc)
	printf("Usage: squaroid [options]\n");
	
	HLP("-h, --help", "print this help text and exit");
	HLP("-v, --version", "print version information and exit");
	HLP("-C, --noconfig", "do not read any configuration file on startup");
	HLP("-s, --server", "set the server to connect to");
	HLP("-S, --nosound", "disable sound temporarily for this session");
	HLP("-c, --connect", "automatically connect on startup");
	HLP("-n, --nick", "set the preferred nickname");
	HLP("", "making squaroid used it's compiled in defaults");
	
	printf("\n");
	printf("The homepage can be found @ http://www.azzit.de/squaroid/\n");
	printf("Report bugs to      <lukas@azzit.de>\n");
}

/* create and display the toplevel window along with menus, notebook,
 * all anything else that's necessary
 */
void sq_create_toplevel_window()
{
  GtkWidget *main_box;
	GdkColormap *colormap;

  if (!blackstyle)
		{
			blackstyle = gtk_style_new();
			blackstyle->font = gdk_font_load("fixed");
			
			greenstyle = gtk_style_new();
			greenstyle->font = gdk_font_load("fixed");
			memcpy(greenstyle->fg, &colors[3], sizeof(GdkColor));
		}
	
  top_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(top_window), "delete_event",
										 GTK_SIGNAL_FUNC(callback_destroy), NULL);
  gtk_window_set_title(GTK_WINDOW(top_window), "Squaroid " VERSION);
  gtk_widget_set_usize(top_window, 780, 530);
  
  colormap = gtk_widget_get_colormap(top_window);
  gdk_color_parse("black", &black);
  gdk_colormap_alloc_color(colormap, &black, FALSE, TRUE);
  gdk_color_parse("white", &white);
  gdk_colormap_alloc_color(colormap, &white, FALSE, TRUE);
	if (font == 0)
		font = gdk_font_load("fixed");
  
  main_box = gtk_vbox_new(0,0);
  top_menubar = createmenus();
  top_notebook = gtk_notebook_new();
  top_statusbar = gtk_statusbar_new();
  gtk_box_pack_start(GTK_BOX(main_box), top_menubar, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(main_box), top_notebook, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(main_box), top_statusbar, FALSE, FALSE, 0);
	
  gtk_signal_connect(GTK_OBJECT(top_notebook), "switch_page",
										 GTK_SIGNAL_FUNC(cb_notebook_switch), NULL);
	
  gtk_container_add(GTK_CONTAINER(top_window), main_box);
  gtk_widget_show_all(main_box);
  gtk_widget_show(top_window);
	
  sq_status_set("Welcome to Squaroid");
}

void sq_notebook_append_page(GtkWidget *container, GtkWidget *label)
{
	int num;
	
  gtk_notebook_append_page(GTK_NOTEBOOK(top_notebook), container, label);
	num = gtk_notebook_page_num(GTK_NOTEBOOK(top_notebook), container);
	gtk_notebook_set_page(GTK_NOTEBOOK(top_notebook), num);
	
	if (!current_tablabel)
		current_tablabel = label;
}

void sig_segv(int sig)
{
	fprintf(stderr, "Caught a segmentation fault (SIGSEGV). Exiting\n");
	sq_finalize_soundserver();
	exit(255);
}

void sig_term(int sig)
{	
	fprintf(stderr, "Caught a termination signal (SIGTERM). Exiting\n");
	sq_finalize_soundserver();
	exit(255);
}

void sig_int(int sig)
{
	fprintf(stderr, "Caught an interrupt (SIGINT). Exiting\n");
	sq_finalize_soundserver();
	exit(255);
}

extern char *optarg;
extern int optind, opterr, optopt;
int main(int argc, char **argv)
{
	int c, option_index;
	int cfg_load_config = 1;
	static struct option long_options[] = 
	{
		{ "noconfig", 0, 0, 'C'},
		{ "version", 0, 0, 'v'},
		{ "help", 0, 0, 'h'},
		{ "connect", 0, 0, 'c'},
		{ "nick", 1, 0, 'n'},
		{ "port", 1, 0, 'p'},
		{ "server", 1, 0, 's'},
		{ "nosound", 0, 0, 'S'},
		{ 0, 0, 0, 0}
	};

#define READ_CFGFILE 0
#define SET_DEFAULTS 1
	
	sq_plugin_init();
	load_config(SET_DEFAULTS);    /* make sure to fill in the defaults */
	opterr = 0;
	while ((c = getopt_long(argc, argv, "Cvcn:s:Sp:P:",
													long_options,	&option_index)) != -1)
		{
			switch(c)
				{
				case 'C':
					cfg_load_config = 0;
					break;
				case 'n':
					strncpy(preferences.nickname, optarg, 63);
					break;
				case 'p':
					preferences.port = strtol(optarg, NULL, 10);
					break;
				case 'P':
					sq_plugin_load(optarg);
					break;
				case 's':
					strncpy(preferences.hostname, optarg, 126);
					break;
				case 'S':
					preferences.sound = 0;
					break;
				case 'v':
					sq_print_version();
					exit(0);
					break;
				case '?':              /* unspecified option character */
				case 'h':
					sq_print_version();
					sq_print_help();
					exit(0);
					break;
				}
		}

	if (cfg_load_config)
		load_config(READ_CFGFILE);

#if SOUND
	/* must initilize sound BEFORE the X connection; otherwise the child
		 inherits the X connection which would be killed along with the
		 playing process to; dont try it, or you'll get ugly Xlib crash
		 results ;) */ 
	if (preferences.sound == 1)
		sq_initialize_soundserver();
	else
		fprintf(stderr, "Sound disabled\n");
#endif
	
	signal(SIGSEGV, sig_segv);
	signal(SIGTERM, sig_term);
	signal(SIGINT, sig_int);
	
  gtk_init(&argc, &argv);
  gdk_imlib_init();
  gtk_widget_push_visual(gdk_imlib_get_visual());
  gtk_widget_push_colormap(gdk_imlib_get_colormap());
	
	sqirc_initialize(server_commands, client_commands, ctcp_commands);
	sqirc_register_callback(IRC_FUNC_CLEANUP, sq_connection_cleanup);
	
	sq_create_toplevel_window();
	ADD_LIST_ITEM(chatroom_list, chatroom_create(0));    /* dummy entry */
	ADD_LIST_ITEM(chatroom_list, chatroom_create(NULL));
#if DEBUG || NET_DEBUG
	chatroom_create_debugwindow();
#endif
	
  if (sq_check_files())
    {
      fprintf(stderr, "Files are missing: fix this and try "
							"again\n"
							"To get a clean release check out \n"
							"   http://www.squaroid.de/\n");
      disable_game = TRUE;
    }
  else
		disable_game = FALSE;
  
  if (!disable_game)
		{
			show_dialog_connect();
		}
	else
		exit(1);
  
  gtk_main();
  return(0);
}
