/* Squaroid
 * Copyright (C) 1999 Lukas Schrder
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "squaroid.h"

static gint cb_timeout_pending_play_request(Game *game);
static gint cb_timeout_pending_observe_request(Game *game);
static gint cb_timeout_movelimit(Game *game);

void timeout_add_pending_play_request(Game *game)
{
	if (game->timeout_play)
		timeout_del_pending_play_request(game);
  game->timeout_play
		= gtk_timeout_add(10*1000,
											(GtkFunction)cb_timeout_pending_play_request,
											game);
}

void timeout_del_pending_play_request(Game *game)
{
	if (game->timeout_play)
		gtk_timeout_remove(game->timeout_play);
  game->timeout_play = 0; /* is 0 invalid? or -1? */
}

void timeout_add_pending_observe_request(Game *game)
{
	if (game->timeout_observe)
		timeout_del_pending_observe_request(game);
  game->timeout_observe
		= gtk_timeout_add(10*1000,
											(GtkFunction)cb_timeout_pending_observe_request,
											game);
}

void timeout_del_pending_observe_request(Game *game)
{
	if (game->timeout_observe)
		gtk_timeout_remove(game->timeout_observe);
  game->timeout_observe = 0; /* is 0 invalid? or -1? */
}

void timeout_add_movelimit(Game *game)
{
	if (game->movelimit.timeout > 0
			|| game->movelimit.mylabel == 0
			|| game->movelimit.opponentlabel == 0
			|| game->movelimit.time_max <= 0)
		return;
	
	game->movelimit.timeout
		= gtk_timeout_add(1*1000,
											(GtkFunction)cb_timeout_movelimit,
											game);
	
	sq_label_set_text(GTK_LABEL(game->movelimit.mylabel), 
										"%d", game->movelimit.time_left);
	sq_label_set_text(GTK_LABEL(game->movelimit.opponentlabel), 
										"%d", game->movelimit.time_left_o);
}

void timeout_del_movelimit(Game *game)
{
	if (game->movelimit.timeout)
		gtk_timeout_remove(game->movelimit.timeout);
	game->movelimit.timeout = 0;
}

/* CALLBACKS */

/* cb_timeout functions are only called if the timeout expired,
 * and we have to print out an error
 */
gint cb_timeout_pending_play_request(Game *game)
{
  timeout_del_pending_play_request(game);
	
	if ( (game->e_opponent->flags & PLAYER_HAVE_VERSION) == 0
			 || (game->e_opponent->flags & PLAYER_HAVE_PLUGINS) == 0
			 || game->e_opponent->valid == FALSE )
		{
			chatroom_chatmessage(CHATROOM(game->connection),
													 STARTON " Invitation to -%s- has timed out!\n",
													 game->opponent);
			sq_notify_wrong_version(game);
		}
	else
		game_play_request(game);
	
  return TRUE;
}

gint cb_timeout_pending_observe_request(Game *game)
{
  timeout_del_pending_observe_request(game);
  chatroom_chatmessage(CHATROOM(game->connection), 
											 STARTON " Observation timeout to -%s-"
											 " has expired!\n", game->opponent);
  sq_notify_wrong_version(game);
  return TRUE;
}

gint cb_timeout_movelimit(Game *game)
{
	if (game->current_player == game->who_am_i)
		{
			game->movelimit.time_left -= 1;
			
			sq_label_set_text(GTK_LABEL(game->movelimit.mylabel), "%d",
												game->movelimit.time_left);
			
			if ((gint)(game->e_opponent->lag*-1) > (gint)game->movelimit.time_left)
				{
					game->game_over = 1;
					timeout_del_movelimit(game);
					chatroom_debugmessage(" movelimit [left=%d] [lag=%d] [%d]\n",
																game->movelimit.time_left,
																game->e_opponent->lag);
					sqirc_send(game->connection,
										 "NOTICE %s :GAMETIMEOUT\r\n", game->e_opponent->nickname);
				}
		}
	else 
		{
			game->movelimit.time_left_o -= 1;
			sq_label_set_text(GTK_LABEL(game->movelimit.opponentlabel), "%d",
												game->movelimit.time_left_o);
		}
	
	return TRUE;
}
