/* X-Chat
 * Copyright (C) 1998 Peter Zelezny.
 * modifications for Squaroid 1999 by Lukas Schrder
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>

#include <gtk/gtkclist.h>
#include <gtk/gtkentry.h>
/* this function cleanly removes a pixmap inside of a clist cell
 * calling gtk_clist_set_pixmap with NULL for pixmap gives you an
 * assertion fault
 */
void gtk_clist_clear_pixmap(GtkCList *clist, int row, int column)
{
  GtkCListRow *clist_row = (g_list_nth(clist->row_list, row))->data;

  g_return_if_fail (clist != NULL);
  g_return_if_fail (GTK_IS_CLIST (clist));

  /* unref'ing several times gives me errors */
  /*
  gdk_pixmap_unref (GTK_CELL_PIXMAP (clist_row->cell[column])->pixmap);
  if (GTK_CELL_PIXMAP (clist_row->cell[column])->mask)
    gdk_bitmap_unref (GTK_CELL_PIXMAP (clist_row->cell[column])->mask);
  */

#define GTK_CLIST_CLASS_FW(_widget_) GTK_CLIST_CLASS (((GtkObject*) (_widget_))->klass)

  GTK_CLIST_CLASS_FW (clist)->set_cell_contents
    (clist, clist_row, column, GTK_CELL_PIXMAP, NULL, 0, 0, 0);

  /* redraw the list if it's not frozen */
  if (clist->freeze_count == 0)
    {
      if (gtk_clist_row_is_visible (clist, row) != GTK_VISIBILITY_NONE)
        GTK_CLIST_CLASS_FW (clist)->draw_row (clist, NULL, row, clist_row);
    }

}

GtkWidget *sq_label_new(char *fmt, ...)
{
	char data[512];
	va_list ap;
	
	va_start(ap, fmt);
	vsnprintf(data, 512, fmt, ap);
	va_end(ap);
	
	return gtk_label_new(data);
}

void sq_label_set_text(GtkLabel *label, char *fmt, ...)
{
	char data[512];
	va_list ap;
	
	va_start(ap, fmt);
	vsnprintf(data, 512, fmt, ap);
	va_end(ap);
	
	gtk_label_set_text(label, data);
}

void sq_entry_set_text(GtkEntry *entry, char *fmt, ...)
{
	char data[512];
	va_list ap;
	
	va_start(ap, fmt);
	vsnprintf(data, 512, fmt, ap);
	va_end(ap);
	
	gtk_entry_set_text(entry, data);
}

#ifdef MEMORY_DEBUG

#define malloc malloc
#define free free

int current_mem_usage;

struct mem_block
{
   void *buf;
   int size;
   struct mem_block *next;
};

struct mem_block *mroot = NULL;

void *
squaroid_malloc (int size)
{
   void *ret;
   struct mem_block *new;

   current_mem_usage += size;
   ret = malloc (size);
   if (!ret)
   {
      printf ("Out of memory! (%d)\n", current_mem_usage);
      exit (253);
   }
   new = malloc (sizeof (struct mem_block));
   new->buf = ret;
   new->size = size;
   new->next = mroot;
   mroot = new;

   printf ("Malloc'ed %d bytes, now %d\n", size, current_mem_usage);

   return ret;
}

void 
squaroid_free (void *buf)
{
   struct mem_block *cur, *last;

   last = NULL;
   cur = mroot;
   while (cur)
   {
      if (buf == cur->buf)
         break;
      last = cur;
      cur = cur->next;
   }
   if (cur == NULL)
   {
      printf ("Tried to free unknown block %x!\n", buf);
      /*      abort(); */
      free (buf);
      free (cur);
      return;
   }
   current_mem_usage -= cur->size;
   printf ("Free'ed %d bytes, usage now %d\n", cur->size, current_mem_usage);
   if (last)
      last->next = cur->next;
   else
      mroot = cur->next;
   free (cur);
}

#endif /* MEMORY_DEBUG */
