#if SOUND

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#ifdef GSM
#	include <gsm.h>
#endif
#include <sys/soundcard.h>
#include "libqdwav/wav_lib.h"
#include "config.h"

wav_sig *file_info;
long count;

/* GSM stuff */

#ifdef GSM

struct {
	gsm handle;
	gsm_frame buf;
	gsm_signal sample[160];
} gsm_info;

#endif

int init_dsp(wav_sig *file_info)
{
	int i, p;
	
	dsp_fd = open(DEFAULT_SFXDEV, O_WRONLY, 0);

	ioctl(dsp_fd, SNDCTL_DSP_RESET, 0);

	p =  file_info->depth;
	i =  ioctl(dsp_fd, SOUND_PCM_WRITE_BITS, &p);

	p =  file_info->chans;
	i += ioctl(dsp_fd, SOUND_PCM_WRITE_CHANNELS, &p);

	p =  file_info->srate;
	i += ioctl(dsp_fd, SOUND_PCM_WRITE_RATE, &p);

	i += ioctl(dsp_fd, SNDCTL_DSP_GETBLKSIZE, &bsize);
	
	ioctl(dsp_fd, SNDCTL_DSP_SYNC, 0);

	return(i);
}

void play_buffer()
{

#ifndef GSM

	if (file_info->len < bsize)
		{
			count = read(file_info->handle, buffer, file_info->len);
			file_info->len -= count;
		} 
	else 
		{
			count = read(file_info->handle, (int *)buffer, bsize);
			file_info->len -= count;
		}
	write(dsp_fd, buffer, count);

#else

	read(file_info->handle, (char *) gsm_info.buf, sizeof(gsm_info.buf));
	gsm_decode(gsm_info.handle, gsm_info.buf, gsm_info.sample);
	count = write(dsp_fd, gsm_info.sample, sizeof(gsm_info.sample));
	file_info->len -= count;
	
	count = 0;
#endif
}

int wav_play_file (char *filename)
{
	long i;
	
	file_info = (wav_sig *)malloc(sizeof(wav_sig));
	buffer = (int *)malloc(32767);
	
	if (init_wav_read(filename, file_info) == -1)
		{
			fprintf(stderr,"Error opening file: %s\n", file_info->name);
			return 0;
		}
	
	if ( (i = init_dsp(file_info)) != 0)
		{
			fprintf(stderr,"Cannot initialize DSP\n");
			return 0;
		}
	
#ifdef GSM	
	gsm_info.handle = gsm_create();
#endif

	while (file_info->len > 0)
			play_buffer();

#ifdef GSM
	gsm_destroy(gsm_info.handle);
#endif
	
	close(dsp_fd);
	close(file_info->handle);

	return 1;
}

#endif /* SOUND */
